@@ -4,10 +4,12 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: config.c,v 1.63.2.2 2008/09/26 19:15:48 phintuka Exp $
+ * $Id: config.c,v 1.115 2012/03/19 11:38:43 phintuka Exp $
  *
  */
 
+#include "features.h"
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -16,10 +18,11 @@
 #include <vdr/config.h>
 #include <vdr/videodir.h>
 #include <vdr/device.h>
+#include <vdr/i18n.h>
 
 #include "logdefs.h"
 #include "config.h"
-#include "i18n.h"
+#include "xine_frontend.h" // HUD_*
 
 #define STRN0CPY(dst, src) \
   do { \
@@ -90,12 +93,6 @@ const char * const config_t::s_deinterlaceMethodNames[ DEINTERLACE_count+1 ] = {
   NULL
 };
 
-const char * const config_t::s_fieldOrder[ FIELD_ORDER_count+1 ] = { 
-  trNOOP("normal"),
-  trNOOP("inverted"),
-  NULL
-};
-
 const char * const config_t::s_audioDrivers[ AUDIO_DRIVER_count+1 ] = { 
   "auto", "alsa", "oss", "none", "esd", "jack",
   NULL
@@ -112,7 +109,18 @@ const char * const config_t::s_audioDriverNames[ AUDIO_DRIVER_count+1 ] = {
 };
 
 const char * const config_t::s_videoDriversX11[ X11_DRIVER_count+1 ] =  {
-  "auto", "xshm", "xv", "xvmc", "xxmc", "vidix", "XDirectFB", "opengl", "sdl", "none",
+  "auto",
+  "xshm",
+  "xv",
+  "vaapi",
+  "vdpau",
+  "opengl",
+  "xvmc",
+  "xxmc",
+  "sdl",
+  "XDirectFB",
+  "vidix",
+  "none",
   NULL
 };
 
@@ -120,12 +128,14 @@ const char * const config_t::s_videoDriverNamesX11[ X11_DRIVER_count+1 ] =  {
   trNOOP("automatic"),
   "XShm",
   "Xv",
+  "VAAPI",
+  "VDPAU",
+  "OpenGL",
   "XvMC",
   "XvMC+VLD",
-  "Vidix",
-  "XDirectFB",
-  "OpenGL",
   "SDL",
+  "XDirectFB",
+  "Vidix",
   trNOOP("no video"),
   NULL
 };
@@ -173,7 +183,7 @@ const char * const config_t::s_audioEqNames[ AUDIO_EQ_count+1 ] = {
 };
 
 const char * const config_t::s_audioVisualizations[ AUDIO_VIS_count+1 ] = {
-  "none", "goom", "oscope", "fftscope", "fftgraph",
+  "none", "goom", "oscope", "fftscope", "fftgraph", "image",
   NULL
 };
 
@@ -183,6 +193,7 @@ const char * const config_t::s_audioVisualizationNames[ AUDIO_VIS_count+1 ] = {
   trNOOP("Oscilloscope"),
   trNOOP("FFT Scope"),
   trNOOP("FFT Graph"),
+  trNOOP("Image"),
   NULL
 };
 
@@ -207,8 +218,8 @@ const char * const config_t::s_subtitleSizes[ SUBTITLESIZE_count+1 ] = {
 };
 
 const char * const config_t::s_subExts[] = {
-  ".sub", ".srt", ".txt", ".ssa",
-  ".SUB", ".SRT", ".TXT", ".SSA", 
+  ".sub", ".srt", ".txt", ".ssa", "ass", "smi",
+  ".SUB", ".SRT", ".TXT", ".SSA", "ASS", "SMI",
   NULL
 };
 
@@ -234,12 +245,148 @@ const char * const config_t::s_osdScalings[] = {
   NULL
 };
 
+const char * const config_t::s_osdColorDepths[] = {
+  trNOOP("automatic"),
+  trNOOP("LUT8"),
+  trNOOP("TrueColor"),
+  NULL
+};
+
+const char * const config_t::s_osdSizes[] = {
+  trNOOP("automatic"),
+  "720x576",
+  "1280x720",
+  "1920x1080",
+  trNOOP("custom"),
+  trNOOP("video stream"),
+  NULL
+};
+
+const char * const config_t::s_decoders_MPEG2[] = {
+  trNOOP("automatic"),
+  "libmpeg2",
+  "FFmpeg",
+  NULL
+};
+
+const char * const config_t::s_decoders_H264[] = {
+  trNOOP("automatic"),
+  "FFmpeg",
+  "CoreAVC",
+  NULL
+};
+
+const char * const config_t::s_ff_skip_loop_filters[] = {
+  trNOOP("automatic"),
+  trNOOP("default"),
+  trNOOP("none"),
+  trNOOP("nonref"),
+  trNOOP("bidir"),
+  trNOOP("nonkey"),
+  trNOOP("all"),
+  NULL
+};
+
+const char * const config_t::s_ff_speed_over_accuracy[] = {
+  trNOOP("automatic"),
+  trNOOP("yes"),
+  trNOOP("no"),
+};
+
+static const char exts_playlist[][4] = {
+  "asx",
+  "m3u",
+  "pls",
+  "ram",
+};
+
+static const char exts_audio[][8] = {
+  "ac3",
+  "asf",
+  "au",
+  "aud",
+  "flac",
+  "mpa",
+  "mpega",
+  "mp2",
+  "mp3",
+  "m4a",
+  "ogg",
+  "ogm",
+  "ra",
+  "spx",
+  "wav",
+  "wma",
+};
+
+static const char exts_video[][8] = {
+  "asf",
+  "avi",
+  "dat",
+  "divx",
+  "dv",
+  "fli",
+  "flv",
+  "iso", /* maybe dvd */
+  "mkv",
+  "mov",
+  "mpeg",
+  "mpg",
+  "mpv",
+  "mp4",
+  "m2v",
+  "m2t",
+  "m2ts",
+  "m4v",
+  "mts",
+  "pes",
+  "rm",
+  "ts",
+  "vdr",
+  "vob",
+  "wmv",
+  "xvid",
+};
+
+static const char exts_image[][8] = {
+  "bmp",
+  "gif",
+  "jpeg",
+  "jpg",
+  "mng",
+  "png",
+  "tiff",
+};
+
+#define DEF_EXT_IS(TYPE) \
+static bool ext_is_ ## TYPE(const char *ext) \
+{ \
+  for(unsigned int i=0; i<sizeof(exts_ ## TYPE)/sizeof(exts_ ## TYPE[0]); i++) \
+    if(!strcasecmp(ext, exts_ ## TYPE[i])) \
+      return true; \
+  return false; \
+} 
+DEF_EXT_IS(playlist)
+DEF_EXT_IS(audio)
+DEF_EXT_IS(video)
+DEF_EXT_IS(image)
+
+static const char *get_extension(const char *fname)
+{
+  if(fname) {
+    const char *pos = strrchr(fname, '.');
+    if(pos)
+      return pos+1;
+  }
+  return NULL;
+}
+
 static char *strcatrealloc(char *dest, const char *src)
 {
   if (!src || !*src) 
     return dest;
 
-  int l = (dest ? strlen(dest) : 0) + strlen(src) + 1;
+  size_t l = (dest ? strlen(dest) : 0) + strlen(src) + 1;
   if(dest) {
     dest = (char *)realloc(dest, l);
     strcat(dest, src);
@@ -250,103 +397,88 @@ static char *strcatrealloc(char *dest, const char *src)
   return dest;
 }
 
+
 bool config_t::IsPlaylistFile(const char *fname)
 {
-  if(fname) {
-    char *pos = strrchr(fname,'.');
-    if(pos) {
-      pos++;
-      if(!strcasecmp(pos, "pls") || 
-	 !strcasecmp(pos, "m3u") || 
-	 !strcasecmp(pos, "ram") ||
-	 !strcasecmp(pos, "asx"))
-	return true;
-    }
-  }
-  return false;
+  const char *ext = get_extension(fname);
+  return ext && ext_is_playlist(ext);
 }
 
 bool config_t::IsAudioFile(const char *fname)
 {
-  if(fname) {
-    char *pos = strrchr(fname,'.');
-    if(pos) {
-      pos++;
-      if(!strcasecmp(pos, "mpa") ||
-	 !strcasecmp(pos, "mp2") ||
-	 !strcasecmp(pos, "mp3") ||
-	 !strcasecmp(pos, "m4a") ||
-	 !strcasecmp(pos, "mpega") ||
-	 !strcasecmp(pos, "flac") ||
-	 !strcasecmp(pos, "ac3") ||
-	 !strcasecmp(pos, "ogg") ||
-	 !strcasecmp(pos, "ogm") ||
-	 !strcasecmp(pos, "au") ||
-	 !strcasecmp(pos, "aud") ||
-	 !strcasecmp(pos, "wma") ||
-	 !strcasecmp(pos, "asf") ||
-	 !strcasecmp(pos, "wav") ||
-	 !strcasecmp(pos, "spx") ||
-	 !strcasecmp(pos, "ra"))
-	return true;
-      return IsPlaylistFile(fname);
-    }
-  }
-  return false;
+  const char *ext = get_extension(fname);
+  return ext && (ext_is_audio(ext) || ext_is_playlist(ext));
 }
 
 bool config_t::IsVideoFile(const char *fname)
 {
-  if(fname) {
-    char *pos = strrchr(fname,'.');
-    if(pos) {
-      pos++;
-      if(!strcasecmp(pos, "avi") ||
-	 !strcasecmp(pos, "mpv") ||
-	 !strcasecmp(pos, "m2v") ||
-	 !strcasecmp(pos, "m4v") ||
-	 !strcasecmp(pos, "vob") ||
-	 !strcasecmp(pos, "vdr") ||
-	 !strcasecmp(pos, "mpg") ||
-	 !strcasecmp(pos, "mpeg")||
-	 !strcasecmp(pos, "mp4") ||
-	 !strcasecmp(pos, "asf") ||
-	 !strcasecmp(pos, "wmv") ||
-	 !strcasecmp(pos, "mov") ||
-	 !strcasecmp(pos, "ts") ||
-	 !strcasecmp(pos, "pes") ||
-	 !strcasecmp(pos, "xvid") ||
-	 !strcasecmp(pos, "divx") ||
-	 !strcasecmp(pos, "fli") ||
-	 !strcasecmp(pos, "flv") ||
-	 !strcasecmp(pos, "dv") ||
-	 !strcasecmp(pos, "dat") ||
-	 !strcasecmp(pos, "mkv") ||
-	 !strcasecmp(pos, "rm") ||
-	 !strcasecmp(pos, "iso"))  /* maybe dvd */
-	return true;
-      return IsAudioFile(fname);
+  const char *ext = get_extension(fname);
+  return ext && (ext_is_video(ext) || ext_is_audio(ext) || ext_is_playlist(ext));
+}
+
+bool config_t::IsImageFile(const char *fname)
+{
+  const char *ext = get_extension(fname);
+  return ext && (ext_is_image(ext) || ext_is_playlist(ext));
+}
+
+bool config_t::IsDvdImage(const char *fname)
+{
+  const char *ext = get_extension(fname);
+  return (ext && !strcasecmp(ext, "iso")) ? true : false;
+}
+
+bool config_t::IsBluRayImage(const char *fname)
+{
+  if (IsDvdImage(fname)) {
+    struct stat st;
+    if (!stat(fname, &st)) {
+      return st.st_size > 10*1024*1024*1024ll;
     }
   }
   return false;
 }
 
-bool config_t::IsImageFile(const char *fname)
+bool config_t::IsDvdFolder(const char *fname)
 {
-  if(fname) {
-    char *pos = strrchr(fname,'.');
-    if(pos) {
-      pos++;
-      if(!strcasecmp(pos, "jpg") ||
-	 !strcasecmp(pos, "jpeg") ||
-	 !strcasecmp(pos, "gif") ||
-	 !strcasecmp(pos, "tiff") || 
-	 !strcasecmp(pos, "bmp") || 
-	 !strcasecmp(pos, "mng") ||
-	 !strcasecmp(pos, "png"))
-	return true;
-    }
+  struct stat st;
+
+  cString folder = cString::sprintf("%s/VIDEO_TS/", fname);
+  if (stat(folder, &st) != 0) {
+    folder = cString::sprintf("%s/video_ts/", fname);
+    if (stat(folder, &st) != 0)
+      return false;
   }
+
+  if (stat(cString::sprintf("%s/video_ts.ifo", *folder), &st) == 0)
+    return true;
+
+  if (stat(cString::sprintf("%s/VIDEO_TS.IFO", *folder), &st) == 0)
+    return true;
+
+  return false;
+}
+
+bool config_t::IsBluRayFolder(const char *fname)
+{
+  struct stat st;
+
+  cString folder = cString::sprintf("%s/BDMV/", fname);
+  if (stat(folder, &st) != 0) {
+    folder = cString::sprintf("%s/bdmv/", fname);
+    if (stat(folder, &st) != 0)
+      return false;
+  }
+
+  if (stat(cString::sprintf("%s/MovieObject.bdmv", *folder), &st) == 0)
+    return true;
+
+  if (stat(cString::sprintf("%s/movieobject.bdmv", *folder), &st) == 0)
+    return true;
+
+  if (stat(cString::sprintf("%s/MOVIEOBJECT.BDMV", *folder), &st) == 0)
+    return true;
+
   return false;
 }
 
@@ -355,8 +487,19 @@ cString config_t::AutocropOptions(void)
   if (!autocrop)
     return NULL;
 
-  return cString::sprintf("enable_autodetect=%d,soft_start=%d,stabilize=%d,enable_subs_detect=%d",
-			  autocrop_autodetect, autocrop_soft, autocrop_fixedsize, autocrop_subs);
+  return cString::sprintf(
+                  "enable_autodetect=%d,autodetect_rate=%d,"
+                  "soft_start=%d,soft_start_step=%d,"
+                  "stabilize=%d,stabilize_time=%d,"
+                  "enable_subs_detect=%d,subs_detect_lifetime=%d,subs_detect_stabilize_time=%d,"
+                  "logo_width=%d,overscan_compensate=%d,use_driver_crop=%d,"
+                  "use_avards_analysis=%d,bar_tone_tolerance=%d",
+			  autocrop_autodetect, autocrop_autodetect_rate,
+			  autocrop_soft, autocrop_soft_start_step,
+			  autocrop_fixedsize, autocrop_stabilize_time,
+			  autocrop_subs, autocrop_subs_detect_lifetime, autocrop_subs_detect_stabilize_time,
+			  autocrop_logo_width, autocrop_overscan_compensate, autocrop_use_driver_crop,
+			  autocrop_use_avards_analysis, autocrop_bar_tone_tolerance);
 }
 
 cString config_t::SwScaleOptions(void)
@@ -419,12 +562,14 @@ config_t::config_t() {
   speaker_type = SPEAKERS_STEREO;
 
   post_plugins = NULL;
+  config_file = NULL;
 
   audio_delay       = 0;
   audio_compression = 0;
   memset(audio_equalizer,0,sizeof(audio_equalizer));
   strn0cpy(audio_visualization, "goom", sizeof(audio_visualization));
   strn0cpy(audio_vis_goom_opts, "fps:25,width:720,height:576", sizeof(audio_vis_goom_opts));
+  strn0cpy(audio_vis_image_mrl, "file:/usr/share/xine/visuals/default.avi", sizeof(audio_vis_image_mrl));
 
   headphone = 0;
   audio_upmix = 0;
@@ -455,37 +600,49 @@ config_t::config_t() {
   display_aspect       = 0;     /* auto */
 
   hide_main_menu       = 0;
+  osd_size             = OSD_SIZE_auto;
+  osd_width            = 720;
+  osd_height           = 576;
+  osd_width_auto       = 0;
+  osd_height_auto      = 0;
+  osd_color_depth      = OSD_DEPTH_auto;
   osd_mixer            = OSD_MIXER_FULL;
   osd_scaling          = OSD_SCALING_NEAREST;
+  osd_spu_scaling      = OSD_SCALING_NEAREST;
   hud_osd              = 0;
+  opengl               = 0;
 
   osd_blending             = OSD_BLENDING_SOFTWARE;
   osd_blending_lowresvideo = OSD_BLENDING_HARDWARE;
 
-#if VDRVERSNUM < 10515
-  spu_autoshow = 0;
-  memset(spu_lang, 0, sizeof(spu_lang));
-  strn0cpy(spu_lang[0], "eng", sizeof(spu_lang[0]));
-  strn0cpy(spu_lang[1], "deu", sizeof(spu_lang[1]));
-  strn0cpy(spu_lang[2], "fin", sizeof(spu_lang[2]));
-  strn0cpy(spu_lang[3], "" ,   sizeof(spu_lang[3]));
-#endif
   extsub_size = -1;
+  dvb_subtitles = 0;
 
   alpha_correction     = 0;
   alpha_correction_abs = 0;
 
   fullscreen     = 0;
-  modeswitch     = 1;
+  modeswitch     = 0;
   width          = 720;
   height         = 576;
   scale_video    = 0;
-  field_order    = 0;
+
   autocrop       = 0;
   autocrop_autodetect = 1;
+  autocrop_autodetect_rate = 4;
   autocrop_soft  = 1;
+  autocrop_soft_start_step  = 4;
   autocrop_fixedsize = 1;
+  autocrop_stabilize_time = (5*25);
   autocrop_subs  = 1;
+  autocrop_subs_detect_lifetime = (60*25);
+  autocrop_subs_detect_stabilize_time = 12;
+  autocrop_logo_width = 20;
+  autocrop_use_driver_crop = 0;
+  autocrop_use_avards_analysis = 0;
+  autocrop_overscan_compensate = 0;
+  autocrop_bar_tone_tolerance = 0;
+
 
   swscale               = 0;    // enable/disable
   swscale_change_aspect = 0;    // change video aspect ratio
@@ -497,6 +654,7 @@ config_t::config_t() {
   remote_mode    = 0;
   listen_port    = LISTEN_PORT;
   remote_keyboard = 1;
+  remote_max_clients = MAXCLIENTS;
   remote_usetcp   = 1;
   remote_useudp   = 1;
   remote_usertp   = 1;
@@ -521,39 +679,51 @@ config_t::config_t() {
   remote_local_ip[0] = 0;   // bind locally to this IP - undefined -> any/all
 
   use_x_keyboard = 1;
+  window_id      = WINDOW_ID_NONE;
 
   // video settings
-#ifdef DEVICE_SUPPORTS_IBP_TRICKSPEED
   ibp_trickspeed = 1;
-#else
-  ibp_trickspeed = 0;
-#endif
   max_trickspeed = 12;
   overscan       = 0;
   hue          = -1; 
   saturation   = -1; 
   contrast     = -1; 
   brightness   = -1; 
+  sharpness    = -1;
+  noise_reduction = -1;
   vo_aspect_ratio = 0;
 
   live_mode_sync = 1;      // Sync SCR to transponder clock in live mode
-  scr_tunning    = 0;      // Fine-tune xine egine SCR (to sync video to graphics output)
+  scr_tuning     = 0;      // Fine-tune xine egine SCR (to sync video to graphics output)
   scr_hz         = 90000;  // Current SCR speed (Hz), default is 90000
 
+  decoder_mpeg2  = DECODER_MPEG2_auto;
+  decoder_h264   = DECODER_H264_auto;
+  ff_h264_speed_over_accurancy = FF_H264_SPEED_OVER_ACCURACY_auto;
+  ff_h264_skip_loop_filter     = FF_H264_SKIP_LOOPFILTER_auto;
+
+  strn0cpy(media_root_dir,    "/",            sizeof(media_root_dir));
   strn0cpy(browse_files_dir,  VideoDirectory, sizeof(browse_files_dir));
   strn0cpy(browse_music_dir,  VideoDirectory, sizeof(browse_music_dir));
   strn0cpy(browse_images_dir, VideoDirectory, sizeof(browse_images_dir));
+  show_hidden_files = 0;
   cache_implicit_playlists = 1;
   enable_id3_scanner = 1;
+  dvd_arrow_keys_control_playback = 1;
+  media_menu_items = ~0;
+  media_enable_delete = 0;
 
   main_menu_mode = ShowMenu;
+  last_hotkey = -1;//kNone;
   force_primary_device = 0;
 };
 
+#if 0
 static uint8_t g_hidden_options[sizeof(config_t)] = {0};
 static uint8_t g_readonly_options[sizeof(config_t)] = {0};
 uint8_t *config_t::hidden_options   = &g_hidden_options[0];
 uint8_t *config_t::readonly_options = &g_readonly_options[0];
+#endif
 
 cString config_t::m_ProcessedArgs;
 bool config_t::ProcessArg(const char *Name, const char *Value)
@@ -567,13 +737,15 @@ bool config_t::ProcessArg(const char *Name, const char *Value)
 
 bool config_t::ProcessArgs(int argc, char *argv[])
 {
-  static const char short_options[] = "fDw:h:l:r:A:V:d:P:pc";
+  static const char short_options[] = "fDw:h:l:mr:A:V:d:P:C:pc";
 
   static const struct option long_options[] = {
       { "fullscreen",   no_argument,       NULL, 'f' },
-      { "hud",          no_argument,       NULL, 'D' },
+      { "hud",          optional_argument, NULL, 'D' },
+      { "opengl",       no_argument,       NULL, 'O' },
       { "width",        required_argument, NULL, 'w' },
       { "height",       required_argument, NULL, 'h' },
+      { "geometry",     required_argument, NULL, 'g' },
       //{ "xkeyboard",    no_argument,       NULL, 'k' },
       //{ "noxkeyboard",  no_argument,       NULL, 'K' },
       { "local",        required_argument, NULL, 'l' },
@@ -582,10 +754,13 @@ bool config_t::ProcessArgs(int argc, char *argv[])
       { "audio",        required_argument, NULL, 'A' },
       { "video",        required_argument, NULL, 'V' },
       { "display",      required_argument, NULL, 'd' },
+      { "window",       required_argument, NULL, 'W' },
+      { "modeswitch",   no_argument,       NULL, 'm' },
       { "post",         required_argument, NULL, 'P' },
+      { "config",       required_argument, NULL, 'C' },
       { "primary",      no_argument,       NULL, 'p' },
-      { "exit-on-close",no_argument,     NULL, 'c' },
-      { NULL }
+      { "exit-on-close",no_argument,       NULL, 'c' },
+      { NULL,           no_argument,       NULL,  0  }
     };
 
   int c;
@@ -593,19 +768,53 @@ bool config_t::ProcessArgs(int argc, char *argv[])
     switch (c) {
     case 'd': ProcessArg("Video.Port", optarg);
               break;
+    case 'W': ProcessArg("X11.WindowId", optarg);
+              break;
+    case 'm': ProcessArg("VideoModeSwitching", "1");
+#ifndef HAVE_XRANDR
+              LOGMSG("Video mode switching not supported");
+#endif
+              break;
     case 'f': ProcessArg("Fullscreen", "1");
               break;
     case 'D': ProcessArg("X11.HUDOSD", "1");
+              if (optarg && strstr(optarg, "xshape")) {
+                ProcessArg("XShapeHUDOSD", "1");
+#ifndef HAVE_XSHAPE
+                LOGMSG("XShape HUD OSD not supported\n");
+#endif
+              }
+              if (optarg && strstr(optarg, "opengl")) {
+                ProcessArg("OpenglHUDOSD", "1");
+#ifndef HAVE_OPENGL
+                LOGMSG("OpenGL HUD OSD not supported\n");
+#endif
+              }
 #ifndef HAVE_XRENDER
-              LOGMSG("HUD OSD not supported\n");
+              else
+                LOGMSG("HUD OSD not supported\n");
 #endif
               break;
-    case 'w': ProcessArg("Fullscreen", "0");
+    case 'O': ProcessArg("OpenglAlways", "1");
+#ifndef HAVE_OPENGL
+              LOGMSG("OpenGL not supported\n");
+#endif
+              break;
+    case 'w': //ProcessArg("Fullscreen", "0");
               ProcessArg("X11.WindowWidth", optarg);
               break;
-    case 'h': ProcessArg("Fullscreen", "0");
+    case 'h': //ProcessArg("Fullscreen", "0");
               ProcessArg("X11.WindowHeight", optarg);
               break;
+    case 'g': {
+                int _width = width, _height = height, _xpos = 0, _ypos = 0;
+                sscanf (optarg, "%dx%d+%d+%d", &_width, &_height, &_xpos, &_ypos);
+                ProcessArg("X11.WindowWidth",  *cString::sprintf("%d", _width));
+                ProcessArg("X11.WindowHeight", *cString::sprintf("%d", _height));
+                ProcessArg("X11.XPos",         *cString::sprintf("%d", _xpos));
+                ProcessArg("X11.YPos",         *cString::sprintf("%d", _ypos));
+              }
+              break;
     //case 'k': ProcessArg("X11.UseKeyboard", "1");
     //          break;
     //case 'K': ProcessArg("X11.UseKeyboard", "0");
@@ -647,6 +856,8 @@ bool config_t::ProcessArgs(int argc, char *argv[])
                 post_plugins = strcatrealloc(post_plugins, ";");
               post_plugins = strcatrealloc(post_plugins, optarg);
               break;
+    case 'C': config_file = strdup(optarg);
+              break;
     case 'p': ProcessArg("ForcePrimaryDevice", "1");
               break;
     case 'c': exit_on_close = 1;
@@ -660,7 +871,7 @@ bool config_t::ProcessArgs(int argc, char *argv[])
 
 bool config_t::SetupParse(const char *Name, const char *Value)
 {
-  char *pt;
+  const char *pt;
   if(*m_ProcessedArgs && NULL != (pt=strstr(m_ProcessedArgs+1, Name)) &&
      *(pt-1) == ' ' && *(pt+strlen(Name)) == ' ') {
     LOGDBG("Skipping configuration entry %s=%s (overridden in command line)", Name, Value);
@@ -674,10 +885,16 @@ bool config_t::SetupParse(const char *Name, const char *Value)
   else if (!strcasecmp(Name, "DisplayAspect"))      display_aspect = strstra(Value, s_aspects, 0);
   else if (!strcasecmp(Name, "ForcePrimaryDevice")) force_primary_device = atoi(Value);
 
+  else if (!strcasecmp(Name, "X11.WindowId"))     window_id = (!strcmp(Value, "root")) ? WINDOW_ID_ROOT : atoi(Value);
   else if (!strcasecmp(Name, "X11.WindowWidth"))  width = atoi(Value);
   else if (!strcasecmp(Name, "X11.WindowHeight")) height = atoi(Value);
+  else if (!strcasecmp(Name, "X11.XPos"))         xpos = atoi(Value);
+  else if (!strcasecmp(Name, "X11.YPos"))         ypos = atoi(Value);
   else if (!strcasecmp(Name, "X11.UseKeyboard"))  use_x_keyboard = atoi(Value);
-  else if (!strcasecmp(Name, "X11.HUDOSD"))       hud_osd = atoi(Value);
+  else if (!strcasecmp(Name, "X11.HUDOSD"))       hud_osd |= (atoi(Value) ? HUD_COMPOSITE : 0);
+  else if (!strcasecmp(Name, "X11.OpenglAlways")) opengl = atoi(Value);
+  else if (!strcasecmp(Name, "X11.OpenglHUDOSD")) hud_osd |= (atoi(Value) ? HUD_OPENGL : 0);
+  else if (!strcasecmp(Name, "X11.XShapeHUDOSD")) hud_osd |= (atoi(Value) ? HUD_XSHAPE : 0);
 
   else if (!strcasecmp(Name, "Audio.Driver")) STRN0CPY(audio_driver, Value);
   else if (!strcasecmp(Name, "Audio.Port"))   STRN0CPY(audio_port, Value);
@@ -686,6 +903,7 @@ bool config_t::SetupParse(const char *Name, const char *Value)
   else if (!strcasecmp(Name, "Audio.Delay"))  audio_delay = atoi(Value);
   else if (!strcasecmp(Name, "Audio.Compression")) audio_compression = atoi(Value);
   else if (!strcasecmp(Name, "Audio.Visualization.GoomOpts")) STRN0CPY(audio_vis_goom_opts, Value);
+  else if (!strcasecmp(Name, "Audio.Visualization.ImageMRL")) STRN0CPY(audio_vis_image_mrl, Value);
   else if (!strcasecmp(Name, "Audio.Visualization")) STRN0CPY(audio_visualization, Value);
   else if (!strcasecmp(Name, "Audio.Surround"))  audio_surround = atoi(Value);
   else if (!strcasecmp(Name, "Audio.Upmix"))     audio_upmix = atoi(Value);
@@ -693,8 +911,13 @@ bool config_t::SetupParse(const char *Name, const char *Value)
   else if (!strcasecmp(Name, "Audio.SoftwareVolumeControl")) sw_volume_control = atoi(Value);
 
   else if (!strcasecmp(Name, "OSD.HideMainMenu"))   hide_main_menu = atoi(Value);
+  else if (!strcasecmp(Name, "OSD.Size"))           osd_size = strstra(Value, s_osdSizes, 0);
+  else if (!strcasecmp(Name, "OSD.Width"))          osd_width = atoi(Value);
+  else if (!strcasecmp(Name, "OSD.Height"))         osd_height = atoi(Value);
+  else if (!strcasecmp(Name, "OSD.ColorDepth"))     osd_color_depth = strstra(Value, s_osdColorDepths, 0);
   else if (!strcasecmp(Name, "OSD.LayersVisible"))  osd_mixer = atoi(Value);
   else if (!strcasecmp(Name, "OSD.Scaling"))        osd_scaling = atoi(Value);
+  else if (!strcasecmp(Name, "OSD.ScalingSPU"))     osd_spu_scaling = atoi(Value);
   else if (!strcasecmp(Name, "OSD.Blending"))       osd_blending = atoi(Value);
   else if (!strcasecmp(Name, "OSD.BlendingLowRes")) osd_blending_lowresvideo = atoi(Value);
 #if 1
@@ -705,18 +928,13 @@ bool config_t::SetupParse(const char *Name, const char *Value)
   else if (!strcasecmp(Name, "OSD.AlphaCorrection"))    alpha_correction = atoi(Value);
   else if (!strcasecmp(Name, "OSD.AlphaCorrectionAbs")) alpha_correction_abs = atoi(Value);
 
-#if VDRVERSNUM < 10515
-  else if (!strcasecmp(Name, "OSD.SpuAutoSelect")) spu_autoshow = atoi(Value);
-  else if (!strcasecmp(Name, "OSD.SpuLang0")) STRN0CPY(spu_lang[0], Value);
-  else if (!strcasecmp(Name, "OSD.SpuLang1")) STRN0CPY(spu_lang[1], Value);
-  else if (!strcasecmp(Name, "OSD.SpuLang2")) STRN0CPY(spu_lang[2], Value);
-  else if (!strcasecmp(Name, "OSD.SpuLang3")) STRN0CPY(spu_lang[3], Value);
-#endif
   else if (!strcasecmp(Name, "OSD.ExtSubSize"))    extsub_size = atoi(Value);
+  else if (!strcasecmp(Name, "OSD.DvbSubtitles"))  dvb_subtitles = atoi(Value);
 
   else if (!strcasecmp(Name, "RemoteMode"))          remote_mode = atoi(Value);
   else if (!strcasecmp(Name, "Remote.ListenPort"))   listen_port = atoi(Value);
   else if (!strcasecmp(Name, "Remote.Keyboard"))     remote_keyboard = atoi(Value);
+  else if (!strcasecmp(Name, "Remote.MaxClients"))   remote_max_clients = atoi(Value);
   else if (!strcasecmp(Name, "Remote.UseTcp"))       remote_usetcp = atoi(Value);
   else if (!strcasecmp(Name, "Remote.UseUdp"))       remote_useudp = atoi(Value);
   else if (!strcasecmp(Name, "Remote.UseRtp"))       remote_usertp = atoi(Value);
@@ -742,15 +960,26 @@ bool config_t::SetupParse(const char *Name, const char *Value)
 
   else if (!strcasecmp(Name, "Video.Driver"))      STRN0CPY(video_driver, Value);
   else if (!strcasecmp(Name, "Video.Port"))        STRN0CPY(video_port, Value);
+
   else if (!strcasecmp(Name, "Video.Scale"))       scale_video = atoi(Value);
   else if (!strcasecmp(Name, "Video.DeinterlaceOptions")) STRN0CPY(deinterlace_opts, Value);
   else if (!strcasecmp(Name, "Video.Deinterlace")) STRN0CPY(deinterlace_method, Value);
-  else if (!strcasecmp(Name, "Video.FieldOrder"))  field_order=atoi(Value)?1:0;
+
   else if (!strcasecmp(Name, "Video.AutoCrop"))    autocrop = atoi(Value);
   else if (!strcasecmp(Name, "Video.AutoCrop.AutoDetect"))   autocrop_autodetect = atoi(Value);
+  else if (!strcasecmp(Name, "Video.AutoCrop.AutoDetectRate"))   autocrop_autodetect_rate = atoi(Value);
   else if (!strcasecmp(Name, "Video.AutoCrop.SoftStart"))    autocrop_soft = atoi(Value);
+  else if (!strcasecmp(Name, "Video.AutoCrop.SoftStartStep"))    autocrop_soft_start_step = atoi(Value);
   else if (!strcasecmp(Name, "Video.AutoCrop.FixedSize"))    autocrop_fixedsize = atoi(Value);
+  else if (!strcasecmp(Name, "Video.AutoCrop.StabilizeTime"))    autocrop_stabilize_time = atoi(Value);
   else if (!strcasecmp(Name, "Video.AutoCrop.DetectSubs"))   autocrop_subs = atoi(Value);
+  else if (!strcasecmp(Name, "Video.AutoCrop.SubsDetectLifetime"))   autocrop_subs_detect_lifetime = atoi(Value);
+  else if (!strcasecmp(Name, "Video.AutoCrop.SubsDetectStabilizeTime"))   autocrop_subs_detect_stabilize_time = atoi(Value);
+  else if (!strcasecmp(Name, "Video.AutoCrop.LogoWidth"))   autocrop_logo_width = atoi(Value);
+  else if (!strcasecmp(Name, "Video.AutoCrop.UseDriverCrop"))   autocrop_use_driver_crop = atoi(Value);
+  else if (!strcasecmp(Name, "Video.AutoCrop.UseAvardsAnalysis"))   autocrop_use_avards_analysis = atoi(Value);
+  else if (!strcasecmp(Name, "Video.AutoCrop.OverscanCompensate"))   autocrop_overscan_compensate = atoi(Value);
+  else if (!strcasecmp(Name, "Video.AutoCrop.BarToneTolerance"))   autocrop_bar_tone_tolerance = atoi(Value);
 
   else if (!strcasecmp(Name, "Video.SwScale"))           swscale = atoi(Value);
   else if (!strcasecmp(Name, "Video.SwScale.Aspect"))    swscale_change_aspect = atoi(Value);
@@ -763,14 +992,22 @@ bool config_t::SetupParse(const char *Name, const char *Value)
   else if (!strcasecmp(Name, "Video.Saturation"))  saturation = atoi(Value);
   else if (!strcasecmp(Name, "Video.Contrast"))    contrast = atoi(Value);
   else if (!strcasecmp(Name, "Video.Brightness"))  brightness = atoi(Value);
+  else if (!strcasecmp(Name, "Video.Sharpness"))   sharpness = atoi(Value);
+  else if (!strcasecmp(Name, "Video.NoiseReduction")) noise_reduction = atoi(Value);
   else if (!strcasecmp(Name, "Video.Overscan"))    overscan = atoi(Value);
   else if (!strcasecmp(Name, "Video.IBPTrickSpeed"))  ibp_trickspeed = atoi(Value);
   else if (!strcasecmp(Name, "Video.MaxTrickSpeed"))  max_trickspeed = atoi(Value);
   else if (!strcasecmp(Name, "Video.AspectRatio"))    vo_aspect_ratio = atoi(Value);
 
+  else if (!strcasecmp(Name, "Video.Decoder.MPEG2"))  decoder_mpeg2 = strstra(Value, s_decoders_MPEG2, 0);
+  else if (!strcasecmp(Name, "Video.Decoder.H264"))   decoder_h264  = strstra(Value, s_decoders_H264,  0);
+  else if (!strcasecmp(Name, "Video.Decoder.H264.SpeedOverAccuracy")) ff_h264_speed_over_accurancy = strstra(Value, s_ff_speed_over_accuracy,  0);
+  else if (!strcasecmp(Name, "Video.Decoder.H264.SkipLoopFilter"))    ff_h264_skip_loop_filter = strstra(Value, s_ff_skip_loop_filters,  0);
+
   else if (!strcasecmp(Name, "Post.pp.Enable"))    ffmpeg_pp = atoi(Value);
   else if (!strcasecmp(Name, "Post.pp.Quality"))   ffmpeg_pp_quality = atoi(Value);
   else if (!strcasecmp(Name, "Post.pp.Mode"))      STRN0CPY(ffmpeg_pp_mode, Value);
+
   else if (!strcasecmp(Name, "Post.unsharp.Enable"))               unsharp                      = atoi(Value);
   else if (!strcasecmp(Name, "Post.unsharp.luma_matrix_width"))    unsharp_luma_matrix_width    = atoi(Value);
   else if (!strcasecmp(Name, "Post.unsharp.luma_matrix_height"))   unsharp_luma_matrix_height   = atoi(Value);
@@ -778,22 +1015,29 @@ bool config_t::SetupParse(const char *Name, const char *Value)
   else if (!strcasecmp(Name, "Post.unsharp.chroma_matrix_width"))  unsharp_chroma_matrix_width  = atoi(Value);
   else if (!strcasecmp(Name, "Post.unsharp.chroma_matrix_height")) unsharp_chroma_matrix_height = atoi(Value);
   else if (!strcasecmp(Name, "Post.unsharp.chroma_amount"))        unsharp_chroma_amount        = atoi(Value);
+
   else if (!strcasecmp(Name, "Post.denoise3d.Enable"))  denoise3d        = atoi(Value);
   else if (!strcasecmp(Name, "Post.denoise3d.luma"))    denoise3d_luma   = atoi(Value);
   else if (!strcasecmp(Name, "Post.denoise3d.chroma"))  denoise3d_chroma = atoi(Value);
   else if (!strcasecmp(Name, "Post.denoise3d.time"))    denoise3d_time   = atoi(Value);
 
+  else if (!strcasecmp(Name, "Media.RootDir"))           STRN0CPY(media_root_dir, Value);
   else if (!strcasecmp(Name, "Media.BrowseFilesDir"))    STRN0CPY(browse_files_dir, Value);
   else if (!strcasecmp(Name, "Media.BrowseMusicDir"))    STRN0CPY(browse_music_dir, Value);
   else if (!strcasecmp(Name, "Media.BrowseImagesDir"))   STRN0CPY(browse_images_dir, Value);
+  else if (!strcasecmp(Name, "Media.ShowHiddenFiles"))   show_hidden_files = atoi(Value);
   else if (!strcasecmp(Name, "Media.CacheImplicitPlaylists")) cache_implicit_playlists = atoi(Value);
   else if (!strcasecmp(Name, "Media.EnableID3Scanner"))  enable_id3_scanner = atoi(Value);
+  else if (!strcasecmp(Name, "Media.DVD.ArrowKeysControlPlayback")) dvd_arrow_keys_control_playback = atoi(Value);
+  else if (!strcasecmp(Name, "Media.MenuItems"))         media_menu_items = atoi(Value);
+  else if (!strcasecmp(Name, "Media.EnableDelete"))      media_enable_delete = atoi(Value);
+
   else if (!strcasecmp(Name, "Playlist.Tracknumber")) playlist_tracknumber = atoi(Value);
   else if (!strcasecmp(Name, "Playlist.Artist"))      playlist_artist = atoi(Value);
   else if (!strcasecmp(Name, "Playlist.Album"))       playlist_album = atoi(Value);
 
   else if (!strcasecmp(Name, "Advanced.LiveModeSync")) live_mode_sync = atoi(Value);
-  else if (!strcasecmp(Name, "Advanced.AdjustSCR"))    scr_tunning = atoi(Value);
+  else if (!strcasecmp(Name, "Advanced.AdjustSCR"))    scr_tuning = atoi(Value);
   else if (!strcasecmp(Name, "Advanced.SCRSpeed"))     scr_hz = atoi(Value);
 
   else if (!strcasecmp(Name, "Audio.Equalizer")) 
