@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: device.h,v 1.34.2.2 2008/12/10 22:56:15 phintuka Exp $
+ * $Id: device.h,v 1.63 2012/03/17 20:14:41 phintuka Exp $
  *
  */
 
@@ -14,28 +14,33 @@
 #include <vdr/config.h>
 #include <vdr/device.h>
 #include <vdr/tools.h>
+#include <vdr/thread.h>
 
 class cXinelibStatusMonitor;
 class cXinelibThread;
 class cChannel;
 class cFunctor;
 
+struct video_size_s;
+struct ts_state_s;
+
 typedef enum {
-  miTitle  = 0,
+  miTitle        = 0,
   miTracknumber  = 1,
-  miArtist = 2,
-  miAlbum  = 3,
-  miDvdTitleNo = 4,
-  miDvdButtons = 5,
-  mi_Count = 6
+  miArtist       = 2,
+  miAlbum        = 3,
+  miDvdTitleNo   = 4,
+  miDvdButtons   = 5,
+  mi_Count       = 6
 } eMetainfoType;
 
-# define ttXSubtitleNone  (-2)
-# define ttXSubtitleAuto  (-1)
+#define ttXSubtitleNone  (-2)
+#define ttXSubtitleAuto  (-1)
 
 #define MAX_METAINFO_LEN 63
+#define MAX_NUM_PIP      16
 
-class cXinelibDevice : public cDevice 
+class cXinelibDevice : public cDevice
 {
 
   // Singleton
@@ -51,24 +56,22 @@ class cXinelibDevice : public cDevice
 
     virtual ~cXinelibDevice();
 
-
   // device start/stop (from cPlugin)
 
   public:
+    bool InitDevice(void);
     bool StartDevice(void);
     void StopDevice(void);
 
-
   // function calls waiting to be executed in VDR main thread context
 
   private:
-    cList<cFunctor> m_MainThreadFunctors; 
+    cList<cFunctor> m_MainThreadFunctors;
     cMutex m_MainThreadLock;
 
   public:
     void MainThreadHook(void);
 
-
   // Primary device switching
 
   private:
@@ -79,28 +82,27 @@ class cXinelibDevice : public cDevice
 
   public:
     virtual void MakePrimaryDevice(bool On);
-    void ForcePrimaryDevice(bool On);
-
+    bool ForcePrimaryDevice(bool On);
 
   // Device capabilities
-  private:
-    bool      m_VDR_TrickSpeedIBP;
 
   public:
-
     virtual bool HasDecoder(void) const { return true; };
     virtual bool CanReplay(void) const { return true; };
+    virtual bool HasIBPTrickSpeed(void) { return xc.ibp_trickspeed; }
 
-    virtual bool HasIBPTrickSpeed(void);
+    bool SupportsTrueColorOSD(void);
 
   // Playback control
 
   private:
     ePlayMode m_PlayMode;
     int       m_TrickSpeed;
-    int64_t   m_TrickSpeedPts;
     int       m_TrickSpeedMode;
+#if VDRVERSNUM < 10705
+    int64_t   m_TrickSpeedPts;
     int       m_TrickSpeedDelay;
+#endif
 
   public:
     virtual bool SetPlayMode(ePlayMode PlayMode);
@@ -114,8 +116,6 @@ class cXinelibDevice : public cDevice
     virtual bool    Flush(int TimeoutMs = 0);
     virtual int64_t GetSTC(void);
 
-    bool UseIBPTrickSpeed(void);
-
   // Video format facilities
 
   public:
@@ -123,38 +123,19 @@ class cXinelibDevice : public cDevice
     virtual void SetVideoFormat(bool VideoFormat16_9);
     virtual eVideoSystem GetVideoSystem(void);
 
+    struct video_size_s *m_VideoSize;
+    struct ts_state_s   *m_tssVideoSize;
+#if VDRVERSNUM >= 10708
+    virtual void GetVideoSize(int &Width, int &Height, double &VideoAspect);
+#endif
+    virtual void GetOsdSize(int &Width, int &Height, double &PixelAspect);
+
   // Track facilities
 
   protected:
     virtual void SetAudioTrackDevice(eTrackType Type);
-
-  private:
-#if VDRVERSNUM < 10515
-    // (DVD) SPU tracks
-    tTrackId m_DvdSpuTrack[64];
-    int      m_CurrentDvdSpuTrack;
-    bool     m_ForcedDvdSpuTrack;
-#endif
-    char     m_MetaInfo[mi_Count][MAX_METAINFO_LEN+1];
-
   public:
-#if VDRVERSNUM < 10515
-    void ClrAvailableDvdSpuTracks(bool NotifyFrontend = true);
-    bool SetAvailableDvdSpuTrack(int Type, const char *lang = NULL, bool Current = false);
-
-    int   NumDvdSpuTracks(void) const;
-    const tTrackId *GetDvdSpuTrack(int Type) const;
-    const char *GetDvdSpuLang(int Type) const;
-
-    int   GetCurrentDvdSpuTrack(void) const { return m_CurrentDvdSpuTrack; }
-    bool  SetCurrentDvdSpuTrack(int Type, bool Force=false);
-    void  EnsureDvdSpuTrack(void);
-#else
     virtual void SetSubtitleTrackDevice(eTrackType Type);
-#endif
-
-    const char *GetMetaInfo(eMetainfoType Type);
-    void        SetMetaInfo(eMetainfoType Type, const char *Value);
 
   // Audio facilities
 
@@ -167,13 +148,11 @@ class cXinelibDevice : public cDevice
     virtual void SetVolumeDevice(int Volume);
     virtual void SetDigitalAudioDevice(bool On);
 
-
   // Image grabbing
 
   public:
-    virtual uchar *GrabImage(int &Size, bool Jpeg = true, 
-			     int Quality = -1, int SizeX = -1, int SizeY = -1);
-
+    virtual uchar *GrabImage(int &Size, bool Jpeg = true,
+                             int Quality = -1, int SizeX = -1, int SizeY = -1);
 
   // SPU decoder
 
@@ -185,7 +164,6 @@ class cXinelibDevice : public cDevice
   public:
     virtual cSpuDecoder *GetSpuDecoder(void);
 
-
   // Messages from StatusMonitor:
 
   private:
@@ -197,13 +175,11 @@ class cXinelibDevice : public cDevice
     void SetReplayMode(void);
     void StopOutput(void);
 
-
   // Osd Commands (from cXinelibOsd)
 
   public:
     void OsdCmd(void *cmd);
 
-
   // Configuration
 
   private:
@@ -212,70 +188,102 @@ class cXinelibDevice : public cDevice
     cXinelibThread        *m_local;
 
   public:
-    void ConfigurePostprocessing(const char *deinterlace_method, 
-				 int audio_delay,
-				 int audio_compression, 
-				 const int *audio_equalizer,
-				 int audio_surround, 
-				 int speaker_type);
-    void ConfigurePostprocessing(const char *name, bool on=true, 
-				 const char *args=NULL);
-    void ConfigureVideo(int hue, int saturation, int brightness, int contrast,
-			int overscan, int vo_aspect_ratio);
+    void ConfigurePostprocessing(const char *deinterlace_method,
+                                 int audio_delay,
+                                 int audio_compression,
+                                 const int *audio_equalizer,
+                                 int audio_surround,
+                                 int speaker_type);
+    void ConfigurePostprocessing(const char *name, bool on = true,
+                                 const char *args = NULL);
+    void ConfigureVideo(int hue, int saturation, int brightness, int sharpness,
+                        int noise_reduction, int contrast, int overscan, int vo_aspect_ratio);
     // local mode:
-    void ConfigureWindow(int fullscreen, int width, int height, 
-			 int modeswitch, const char *modeline, 
-			 int aspect, int scale_video, int field_order);
+    void ConfigureWindow(int fullscreen, int width, int height,
+                         int modeswitch, const char *modeline,
+                         int aspect, int scale_video);
     void ConfigureDecoder(int pes_buffers);
     // remote mode:
     void Listen(bool activate, int port);
 
-
   // File playback
 
   private:
     ePlayMode m_PlayingFile;
 
   public:
-    bool PlayFile(const char *Filename, int Position=0, 
-		  bool LoopPlay=false, ePlayMode PlayMode=pmAudioVideo);
+    bool PlayFile(const char *Filename, int Position = 0,
+                  bool LoopPlay = false, ePlayMode PlayMode = pmAudioVideo,
+                  int TimeoutMs = -1);
     int  PlayFileCtrl(const char *Cmd, int TimeoutMs = -1);
     bool EndOfStreamReached(void);
 
+  // Metainfo cache
+
+  private:
+    char     m_MetaInfo[mi_Count][MAX_METAINFO_LEN+1];
+
+  public:
+    const char *GetMetaInfo(eMetainfoType Type);
+    void        SetMetaInfo(eMetainfoType Type, const char *Value);
 
   // Stream data
 
   private:
-    bool m_ac3Present;
-    bool m_spuPresent;
     bool m_RadioStream;
     int  m_AudioCount;
     bool m_SkipAudio;
     bool m_StreamStart;
     int  m_FreeBufs;
-    bool m_Cleared;
-    bool m_h264;
 
     int PlayAny(const uchar *Data, int Length);
     int PlayTrickSpeed(const uchar *buf, int length);
 
+    bool AcceptVideoPacket(const uchar *Data, int Length);
+    bool AcceptAudioPacket(const uchar *Data, int Length);
+
   protected:
 
     virtual bool Poll(cPoller &Poller, int TimeoutMs = 0);
 
     virtual void StillPicture(const uchar *Data, int Length);
 
+    // MPEG-PES
     virtual int  PlayVideo(const uchar *Data, int Length);
     virtual int  PlayAudio(const uchar *Data, int Length, uchar Id);
+    virtual int  PlaySubtitle(const uchar *Data, int Length);
 
-#if VDRVERSNUM < 10510
-    virtual int  PlaySpu(const uchar *Data, int Length, uchar Id);
+#if VDRVERSNUM >= 10701 || defined(TSPLAY_PATCH_VERSION)
+    // join multiple TS packets to xineliboutput transport packet
+    cMutex        m_TsBufLock;
+    uint8_t       m_TsBuf[4096];
+    uint          m_TsBufSize;
 
-    // conflicts with vdr-1.5.10+ DVB subtitle handling
-    // override cDevice to get DVD SPUs
-    virtual int PlayPesPacket(const uchar *Data, int Length,
-			      bool VideoOnly = false);
+    int           TsBufferFlush(void);
+    void          TsBufferClear(void);
+
+    int           PlayTsAny(const uchar *Data, int Length);
+
+    // MPEG-TS
+    virtual int PlayTsVideo(const uchar *Data, int Length);
+    virtual int PlayTsAudio(const uchar *Data, int Length);
+    virtual int PlayTsSubtitle(const uchar *Data, int Length);
+    virtual int PlayTs(const uchar *Data, int Length, bool VideoOnly = false);
+#else
+    void        TsBufferClear(void) {}
+    void        TsBufferFlush(void) {}
 #endif
+
+  // Picture-In-Picture
+
+  protected:
+    uint16_t m_PipPid[MAX_NUM_PIP];
+
+  public:
+    int  Pip_Open   (void);
+    void Pip_Config (int Index, int X, int Y, int W, int H);
+    int  Pip_Play   (int Index, uint8_t *Data, int Length);
+    void Pip_Close  (int Index);
 };
 
 #endif // __XINELIB_DEVICE_H
