@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: equalizer.c,v 1.4 2007/10/15 00:15:07 phintuka Exp $
+ * $Id: equalizer.c,v 1.8 2012/03/18 19:17:07 phintuka Exp $
  *
  */
 
@@ -14,8 +14,9 @@
 #include "device.h"
 #include "equalizer.h"
 
-cEqualizer::cEqualizer() : cOsdObject()
+cEqualizer::cEqualizer(cXinelibDevice *Dev) : cOsdObject()
 {
+  m_Dev = Dev;
   m_Values = new int[AUDIO_EQ_count];
   memcpy(m_Values, xc.audio_equalizer, sizeof(xc.audio_equalizer));
   m_Osd = NULL;
@@ -25,7 +26,7 @@ cEqualizer::cEqualizer() : cOsdObject()
 cEqualizer::~cEqualizer()
 {
   delete m_Osd;
-  delete m_Values;
+  delete[] m_Values;
 }
 
 #define OSD_W (220)
@@ -42,11 +43,7 @@ void cEqualizer::Show()
 {
   tArea areas [] = { {0, 0, OSD_W - 1, OSD_H - 1, 4} };
 
-#if VDRVERSNUM >= 10509
   m_Osd = cOsdProvider::NewOsd(OSD_X, OSD_Y, 0);
-#else
-  m_Osd = cOsdProvider::NewOsd(OSD_X, OSD_Y);
-#endif
 
   if(m_Osd) {
     if (m_Osd->CanHandleAreas(areas, sizeof(areas) / sizeof(tArea) ) == oeOk) {
@@ -70,14 +67,14 @@ eOSState cEqualizer::ProcessKey(eKeys key)
 	if(m_Values[m_Current] < ADJUST_MIN)
 	  m_Values[m_Current] = ADJUST_MIN;
 	DrawBar(m_Current,true);
-	cXinelibDevice::Instance().ConfigurePostprocessing(xc.deinterlace_method, xc.audio_delay, xc.audio_compression, m_Values, xc.audio_surround, xc.speaker_type);
+	m_Dev->ConfigurePostprocessing(xc.deinterlace_method, xc.audio_delay, xc.audio_compression, m_Values, xc.audio_surround, xc.speaker_type);
 	break;
       case kUp:
 	m_Values[m_Current] += ADJUST_STEP;
 	if(m_Values[m_Current] > ADJUST_MAX)
 	  m_Values[m_Current] = ADJUST_MAX;
 	DrawBar(m_Current,true);
-	cXinelibDevice::Instance().ConfigurePostprocessing(xc.deinterlace_method, xc.audio_delay, xc.audio_compression, m_Values, xc.audio_surround, xc.speaker_type);
+	m_Dev->ConfigurePostprocessing(xc.deinterlace_method, xc.audio_delay, xc.audio_compression, m_Values, xc.audio_surround, xc.speaker_type);
 	break;
       case kLeft: 
 	if(m_Current>0) {
@@ -94,11 +91,12 @@ eOSState cEqualizer::ProcessKey(eKeys key)
 	}
 	break;
       case kBack:
-        cXinelibDevice::Instance().ConfigurePostprocessing(xc.deinterlace_method, xc.audio_delay, xc.audio_compression, xc.audio_equalizer, xc.audio_surround, xc.speaker_type);
+        m_Dev->ConfigurePostprocessing(xc.deinterlace_method, xc.audio_delay, xc.audio_compression, xc.audio_equalizer, xc.audio_surround, xc.speaker_type);
 	return osEnd;
       case kOk:
         memcpy(xc.audio_equalizer, m_Values, sizeof(xc.audio_equalizer));
 	return osEnd;
+      default:; // all other keys - do nothing.
     }
   }
 
