@@ -4,10 +4,12 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: frontend_local.c,v 1.31 2008/04/03 08:08:27 phintuka Exp $
+ * $Id: frontend_local.c,v 1.61 2013/08/18 07:58:29 phintuka Exp $
  *
  */
 
+#define __STDC_CONSTANT_MACROS
+
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
@@ -17,9 +19,9 @@
 
 #include <vdr/config.h>
 #include <vdr/tools.h>
-#if VDRVERSNUM >= 10501 || (defined(PATCH_SHUTDOWN_REWRITE) && PATCH_SHUTDOWN_REWRITE >= 100)
 #include <vdr/shutdown.h>
-#endif
+#include <vdr/plugin.h>
+#include <vdr/device.h>
 
 #include "logdefs.h"
 #include "config.h"
@@ -49,21 +51,26 @@ class cRwLockBlock
 //----------------- keyboard control handler (C callback) --------------------
 
 extern "C" {
-  static void keypress_handler(const char *keymap, const char *key)
+  static void keypress_handler(void *handle, const char *keymap, const char *key)
   {
-    if(!strncmp("INFO ", keymap, 5)) {
-      
-      cXinelibThread::InfoHandler(keymap+5);
+    if (!handle) {
+      LOGMSG("keypress_handler(): missing handle");
+      return;
+    }
 
-    } else if(!xc.use_x_keyboard || !key) {
+    cXinelibThread *t = (cXinelibThread *)handle;
+
+    if (!strncmp("INFO ", keymap, 5)) {
+      t->InfoHandler(keymap+5);
+
+    } else if (!xc.use_x_keyboard || !key) {
 
       /* Only X11 key events came this way in local mode.
 	 Keyboard is handled by vdr. */
       LOGMSG("keypress_handler(%s): X11 Keyboard disabled in config", key);
 
     } else {
-
-      cXinelibThread::KeypressHandler(keymap, key, false, false);
+      t->KeypressHandler(keymap, key, false, false);
 
     }
   }
@@ -71,35 +78,27 @@ extern "C" {
 
 //----------------------------- cXinelibLocal --------------------------------
 
-cXinelibLocal::cXinelibLocal(const char *frontend_name) :
-  cXinelibThread("Local decoder/display (cXinelibThread)"), m_feLock(true) 
+cXinelibLocal::cXinelibLocal(cXinelibDevice *Dev, const char *frontend_name) :
+  cXinelibThread(Dev, "Local decoder/display (cXinelibThread)"), m_feLock(true)
 {
   fe = NULL;
   h_fe_lib = NULL;
   m_bReconfigRequest = true;
+
+  if (!xc.config_file &&
+      0 < asprintf(&xc.config_file,
+                   "%s/xineliboutput/config",
+                   cPlugin::ConfigDirectory()))
+    LOGMSG("cXinelibLocal: Using xine-lib configuration file %s", xc.config_file);
 }
 
-cXinelibLocal::~cXinelibLocal() 
+cXinelibLocal::~cXinelibLocal()
 {
   TRACEF("cXinelibLocal::~cXinelibLocal");
-  
+
   m_bReady = false;
 
-  Stop();
-  if(fe) {
-    fe->fe_free(fe);
-    fe = NULL;
-  }
-  if(h_fe_lib) {
-    dlclose(h_fe_lib);
-  }
-}
-
-void cXinelibLocal::Stop(void)
-{
-  TRACEF("cXinelibLocal::Stop");
-
-  SetStopSignal();
+  Cancel(-1);
 
   {
     LOCK_FE;
@@ -108,27 +107,31 @@ void cXinelibLocal::Stop(void)
       fe->fe_interrupt(fe);
   }
 
-  cXinelibThread::Stop();
+  Cancel(3);
+
+  if (fe) {
+    fe->fe_free(fe);
+    fe = NULL;
+  }
+  if (h_fe_lib) {
+    dlclose(h_fe_lib);
+  }
 }
 
 //
 // Data transfer
 //
 
-int cXinelibLocal::Play_PES(const uchar *data, int len)
+int cXinelibLocal::Play(const uchar *data, int len, eStreamId StreamId)
 {
-  TRACEF("cXinelibLocal::Play_PES");
+  TRACEF("cXinelibLocal::Play");
 
   {
     LOCK_FE;
-    if(fe && !m_bStopThread) {
-      int done = fe->xine_queue_pes_packet(fe, (char*)data, len);
-      if(done>0) {
-	Lock();
-	m_StreamPos += done;
-	Unlock();
-	return done;
-      }
+    if (fe && Running()) {
+      int done = fe->xine_queue_pes_packet(fe, StreamId, m_StreamPos, (const char *)data, len);
+      if (done >= 0)
+        return cXinelibThread::Play(data, done, StreamId);
     }
   }
 
@@ -160,17 +163,18 @@ int64_t cXinelibLocal::GetSTC()
 {
   TRACEF("cXinelibLocal::GetSTC");
 
-  int64_t pts = -1;
-  char buf[32] = {0};
-  strcpy(buf, "GETSTC\r\n");
+  union {
+    char    buf[32];
+    int64_t pts;
+  } u = {"GETSTC\r\n"};
 
   LOCK_FE;
-  if(fe && m_bReady)
-    if(0 == fe->xine_control(fe, (char*)buf))
-      //if(*((int64_t *)buf) < MAX_SCR)
-      //  if(*((int64_t *)buf) >= 0LL)
-	  pts = *((int64_t *)buf);
-  return pts;
+
+  if (fe && m_bReady)
+    if (0 == fe->xine_control(fe, u.buf))
+      return u.pts;
+
+  return INT64_C(-1);
 }
 
 //
@@ -191,13 +195,13 @@ bool cXinelibLocal::EndOfStreamReached(void)
 
 void cXinelibLocal::ConfigureWindow(int fullscreen, int width, int height, 
 				    int modeswitch, const char *modeline, 
-				    int aspect, int scale_video, 
-				    int field_order) 
+				    int aspect, int scale_video)
 {
   LOCK_FE;
   if(fe)
-    fe->fe_display_config(fe, width, height, fullscreen, modeswitch, modeline, 
-			  aspect, scale_video, field_order);
+    fe->fe_display_config(fe, -1, -1, width, height,
+                          fullscreen, modeswitch, modeline,
+                          aspect, scale_video);
 }
 
 void cXinelibLocal::ConfigureDecoder(int pes_buffers)
@@ -213,12 +217,17 @@ void cXinelibLocal::ConfigureDecoder(int pes_buffers)
     fe->fe_interrupt(fe);    
   }
 
-  while(!m_bReady && !GetStopSignal())
+  while (!m_bReady && Running())
     cCondWait::SleepMs(100);
 
   cCondWait::SleepMs(100);
 }
 
+int cXinelibLocal::SupportsTrueColorOSD(void)
+{
+  return !!xc.hud_osd;
+}
+
 //
 // Xine control
 //
@@ -226,7 +235,7 @@ void cXinelibLocal::ConfigureDecoder(int pes_buffers)
 int cXinelibLocal::Xine_Control(const char *cmd)
 {
   TRACEF("cXinelibLocal::Xine_Control");
-  if(cmd && *cmd && !GetStopSignal()) {
+  if (cmd && *cmd && Running()) {
     char buf[4096];
     if(snprintf(buf, sizeof(buf), "%s\r\n", cmd) >= (int)sizeof(buf)) {
       buf[sizeof(buf)-1] = 0;
@@ -253,24 +262,24 @@ frontend_t *cXinelibLocal::load_frontend(const char *fe_name)
   fe_creator_f *fe_creator = NULL;
   static int my_marker = 0;
 
-  if(!dladdr((void *)&my_marker, &info)) {
-    LOGERR("Error searching plugin: dladdr() returned false (%s)",dlerror());
+  if (!dladdr((void *)&my_marker, &info)) {
+    LOGERR("Error searching plugin: dladdr() returned false (%s)", dlerror());
     return NULL;
   }
   LOGDBG("xineliboutput: plugin file is %s", info.dli_fname);
 
   int  fe_ind = strstra(fe_name, xc.s_frontends, FRONTEND_NONE);
   bool fe_try = false;
-  if(fe_ind == FRONTEND_NONE) {
+  if (fe_ind == FRONTEND_NONE) {
     LOGMSG("Front-end %s unknown!", fe_name);
     fe_ind = 0;
     fe_try = true;
   }
 
   strn0cpy(libname, info.dli_fname, sizeof(libname) - 128);
-  if(strrchr(libname, '/')) 
+  if (strrchr(libname, '/'))
     *(strrchr(libname, '/')+1) = 0;
-  
+
   LOGDBG("Searching frontend %s from %s", xc.s_frontends[fe_ind], libname);
 
   do {
@@ -278,29 +287,28 @@ frontend_t *cXinelibLocal::load_frontend(const char *fe_name)
     LOGDBG("Probing %s", libname);
 
     if (stat(libname, &statbuffer)) {
-      LOGERR("load_frontend: can't stat %s",libname);
+      LOGERR("load_frontend: can't stat %s", libname);
     } else if((statbuffer.st_mode & S_IFMT) != S_IFREG) {
       LOGMSG("load_frontend: %s not regular file ! trying to load anyway ...",
 	     libname);
     }
 
-    if( !(lib = dlopen (libname, RTLD_LAZY | RTLD_GLOBAL))) {
-      char *dl_error_msg = dlerror();
-      LOGERR("load_frontend: cannot dlopen file %s: %s", 
-	     libname, dl_error_msg);
+    if ( !(lib = dlopen (libname, RTLD_LAZY | RTLD_GLOBAL))) {
+      LOGERR("load_frontend: cannot dlopen file %s: %s", libname, dlerror());
+
     } else if ( (fe_creator = (fe_creator_f*)dlsym(lib, "fe_creator"))) {
       LOGDBG("load_frontend: entry at %p", fe_creator);
       frontend_t *fe = (**fe_creator)();
 
-      if(fe) {
-	if(h_fe_lib)
+      if (fe) {
+	if (h_fe_lib)
 	  dlclose(h_fe_lib);
 	h_fe_lib = lib;
 
-	LOGDBG("Using frontend %s (%s) from %s", 
-	       xc.s_frontends[fe_ind], xc.s_frontendNames[fe_ind], 
+	LOGDBG("Using frontend %s (%s) from %s",
+	       xc.s_frontends[fe_ind], xc.s_frontendNames[fe_ind],
 	       xc.s_frontend_files[fe_ind]);
-	
+
 	return fe;
       } else {
 	LOGMSG("Frontend %s (%s) creation failed",
@@ -310,12 +318,12 @@ frontend_t *cXinelibLocal::load_frontend(const char *fe_name)
       LOGERR("Frontend entry point not found");
       dlclose(lib);
     }
-    
+
     fe_ind++;  // try next frontend ...
 
-  } while(fe_try && fe_ind < FRONTEND_count);
+  } while (fe_try && fe_ind < FRONTEND_count);
 
-  LOGMSG("No usable frontends found, giving up !");  
+  LOGMSG("No usable frontends found, giving up !");
   return NULL;
 }
 
@@ -336,17 +344,23 @@ void cXinelibLocal::Action(void)
     curr_fe = load_frontend(xc.local_frontend);
     if(!curr_fe) {
       LOGMSG("cXinelibLocal: Error initializing frontend");
-      SetStopSignal();
+      Cancel(-1);
     } else {
       LOGDBG("cXinelibLocal::Action - fe created");
-      if(!curr_fe->fe_display_open(curr_fe, xc.width, xc.height, xc.fullscreen, xc.hud_osd,
-				   xc.modeswitch, xc.modeline, 
-				   xc.display_aspect, keypress_handler, 
-				   xc.video_port, 
-				   xc.scale_video, 
-				   xc.field_order)) {
+      curr_fe->fe_message_cb = keypress_handler;
+      curr_fe->fe_message_h  = this;
+      if(!curr_fe->fe_display_open(curr_fe,
+                                   xc.xpos, xc.ypos, xc.width, xc.height, xc.fullscreen,
+                                   xc.hud_osd,
+                                   xc.opengl,
+                                   xc.modeswitch, xc.modeline, xc.display_aspect,
+                                   0/*no_x_kbd*/, 0/*gui_hotkeys*/, 0/*touchscreen*/,
+                                   xc.video_port,
+                                   xc.scale_video,
+                                   NULL,
+                                   xc.window_id)) {
 	LOGMSG("cXinelibLocal: Error initializing display");
-	SetStopSignal();
+        Cancel(-1);
       } else {
 	LOGDBG("cXinelibLocal::Action - fe->fe_display_open ok");
       }
@@ -354,19 +368,19 @@ void cXinelibLocal::Action(void)
   }
 
   // main loop
-  while (!GetStopSignal()) {
+  while (Running()) {
 
     {
       // init and start xine engine
       LOCK_FE_WR;
       LOGDBG("cXinelibLocal::Action - xine_init");
-      
+
       fe = curr_fe;
       if(m_bReconfigRequest) {
 	if(!fe->xine_init(fe, xc.audio_driver, xc.audio_port,
 			  xc.video_driver,
 			  xc.pes_buffers,
-			  xc.post_plugins)) {
+			  xc.post_plugins, xc.config_file)) {
 	  LOGMSG("cXinelibLocal: Error initializing frontend");
 	  break;
 	}
@@ -416,9 +430,9 @@ void cXinelibLocal::Action(void)
     LOGDBG("cXinelibLocal:Action - Starting event loop");   
     {
       LOCK_FE;
-      while(!GetStopSignal() && m_bReady && 
-	    (/*m_bLoopPlay ||*/ !fe->xine_is_finished(fe, 0)) && 
-	    fe->fe_run(fe)) 
+      while (Running() && m_bReady &&
+             (/*m_bLoopPlay ||*/ !fe->xine_is_finished(fe, 0)) &&
+             fe->fe_run(fe))
 	/*cCondWait::SleepMs(50)*/ ;
     }
 
@@ -442,22 +456,19 @@ void cXinelibLocal::Action(void)
 
     if(!m_bReconfigRequest && xc.exit_on_close) {
       LOGMSG("Shutting down VDR");
-#if VDRVERSNUM >= 10501 || (defined(PATCH_SHUTDOWN_REWRITE) && PATCH_SHUTDOWN_REWRITE >= 100)
       ShutdownHandler.RequestEmergencyExit();
-#else
-      cThread::EmergencyExit(true);
-#endif
       break;
     }
   }
 
   if(curr_fe) {
-    curr_fe->xine_exit(fe);
+    LOCK_FE_WR;
+    fe = NULL;
+    curr_fe->xine_exit(curr_fe);
     curr_fe->fe_display_close(curr_fe);
     curr_fe->fe_free(curr_fe);
   }
 
-  m_bIsFinished = true;
   LOGMSG("cXinelibLocal::Action: thread finished");
 }
 
