@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: menu.h,v 1.6 2007/01/04 08:42:47 phintuka Exp $
+ * $Id: menu.h,v 1.11 2012/03/19 11:29:14 phintuka Exp $
  *
  */
 
@@ -13,41 +13,38 @@
 
 #include <vdr/menuitems.h>
 
-class cMenuXinelib : public cMenuSetupPage 
+class cXinelibDevice;
+
+class cMenuXinelib : public cMenuSetupPage
 {
   private:
-    int field_order;
+    cXinelibDevice *m_Dev;
+
     int compression;
-    int headphone;
     int autocrop;
     int overscan;
     int novideo;
 
     // Hotkeys
-    enum {hkInit, hkSeen, hkNone} hotkey_state;
-    static time_t g_LastHotkeyTime;
-    static eKeys  g_LastHotkey;
+    enum { hkInit, hkSeen, hkNone } hotkey_state;
+
     virtual eOSState ProcessHotkey(eKeys Key);
 
-#ifdef HAVE_XV_FIELD_ORDER
-    cOsdItem *video_ctrl_interlace_order;
-#endif
     cOsdItem *audio_ctrl_compress;
 
     cOsdItem *ctrl_autocrop;
     cOsdItem *ctrl_overscan;
-    cOsdItem *ctrl_headphone;
     cOsdItem *ctrl_novideo;
- 
+
   protected:
     virtual void Store(void);
 
   public:
-    cMenuXinelib(void);
+    cMenuXinelib(cXinelibDevice *Dev);
     virtual ~cMenuXinelib();
     virtual eOSState ProcessKey(eKeys Key);
 
-    static cOsdMenu *CreateMenuBrowseFiles(eMainMenuMode mode, bool Queue=true);
+    static cOsdMenu *CreateMenuBrowseFiles(cXinelibDevice *Dev, eMainMenuMode mode, bool Queue=true);
 };
 
 #endif //__XINELIB_SETUP_MENU_H
