@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: iso639.h,v 1.1.2.1 2008/09/26 13:49:45 phintuka Exp $
+ * $Id: iso639.h,v 1.6 2011/03/19 17:03:56 phintuka Exp $
  *
  */
 
@@ -60,6 +60,7 @@ static const struct {
   {"guj", "gu"},
   {"hau", "ha"},
   {"heb", "he"},
+  {"heb", "iw"},
   {"hin", "hi"},
   {"hun", "hu"},
   {"ice", "is"},
@@ -154,28 +155,14 @@ static const struct {
   {"zul", "zu"},
 };
 
-static const char *iso639_2_to_iso639_1(const char *lang)
-{
-  if (lang && lang[0]) {
-    if(lang[1] && !lang[2]) {
-      for (unsigned int i = 0 ; i < sizeof(ISO639_map) / sizeof(ISO639_map[0]); i++)
-	if (((uint16_t*)ISO639_map[i].iso639_1)[0] == ((uint16_t*)lang)[0])
-	  return ISO639_map[i].iso639_2;
-      LOGMSG("Unknown iso639-2 code: %s", lang);
-    }
-    return lang;
-  }
-  return NULL;
-}
-
 static const char *iso639_1_to_iso639_2(const char *lang)
 {
   if (lang && lang[0]) {
-    if(lang[1] && lang[2] && !lang[3]) {
-      for (unsigned int i = 0 ; i < sizeof(ISO639_map) / sizeof(ISO639_map[0]); i++)
-	if (((uint16_t*)ISO639_map[i].iso639_2)[0] == ((uint16_t*)lang)[0] &&
-	    ((uint8_t *)ISO639_map[i].iso639_2)[2] == ((uint8_t *)lang)[2])
-	  return ISO639_map[i].iso639_1;
+    if (lang[1] && !lang[2]) {
+      unsigned int i;
+      for (i = 0 ; i < sizeof(ISO639_map) / sizeof(ISO639_map[0]); i++)
+	if (!memcmp(ISO639_map[i].iso639_1, lang, 2))
+	  return ISO639_map[i].iso639_2;
       LOGMSG("Unknown iso639-1 code: %s", lang);
     }
     return lang;
@@ -183,4 +170,19 @@ static const char *iso639_1_to_iso639_2(const char *lang)
   return NULL;
 }
 
+static const char *iso639_2_to_iso639_1(const char *lang)
+{
+  if (lang && lang[0]) {
+    if (lang[1] && lang[2] && !lang[3]) {
+      unsigned int i;
+      for (i = 0 ; i < sizeof(ISO639_map) / sizeof(ISO639_map[0]); i++)
+	if (!memcmp(ISO639_map[i].iso639_2, lang, 3))
+	  return ISO639_map[i].iso639_1;
+      LOGMSG("Unknown iso639-2 code: %s", lang);
+    }
+    return lang;
+  }
+  return NULL;
+}
+
 #endif
