@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: playlist.h,v 1.8 2008/02/19 04:24:34 phintuka Exp $
+ * $Id: playlist.h,v 1.14 2010/12/09 14:13:03 phintuka Exp $
  *
  */
 
@@ -89,12 +89,11 @@ class cPlaylist : protected cList<cPlaylistItem>
   int ReadPlaylist(const char *PlaylistFile);
   int ScanFolder(const char *FolderName, 
 		 bool Recursive = false,
-		 bool (config_t::*Filter)(const char *) = &config_t::IsAudioFile);
+		 bool (*Filter)(const char *) = &config_t::IsAudioFile);
 
   friend class cID3Scanner;
   friend class cPlaylistReader;
   void PlaylistChanged(const cPlaylistItem *Item);
-  cPlaylistItem *Last(void) { return cList<cPlaylistItem>::Last(); }
 
  public:
 
@@ -110,6 +109,7 @@ class cPlaylist : protected cList<cPlaylistItem>
   bool Read(const char *PlaylistFile, bool Recursive = false);
   void StartScanner(void);
   void Del(cPlaylistItem *it);
+  void Move(int From, int To);
 
   void Sort(void);
   int  Count(void) const;
@@ -117,14 +117,17 @@ class cPlaylist : protected cList<cPlaylistItem>
   // access/iterate playlist items
   cPlaylistItem *First(void) { return Next(NULL); }
   cPlaylistItem *Next(const cPlaylistItem *i);
+  cPlaylistItem *Last(void) { return cList<cPlaylistItem>::Last(); }
 
   // get/set current (now playing) item
   cPlaylistItem *Current(void);
   void           SetCurrent(cPlaylistItem *current);
   cPlaylistItem *Next(void);
   cPlaylistItem *Prev(void);
+  cPlaylistItem *Seek(int Rel);
 
-  static cString EscapeMrl(const char *name);
+  static cString BuildMrl(const char *proto, const char *s1,
+                          const char *s2 = NULL, const char *s3 = NULL, const char *s4 = NULL);
   static cString GetEntry(cPlaylistItem *i, bool isPlaylist = false, bool isCurrent = false);
 };
 
