@@ -4,10 +4,14 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: udp_pes_scheduler.c,v 1.34.2.1 2009/02/12 11:18:37 phintuka Exp $
+ * $Id: udp_pes_scheduler.c,v 1.56 2011/02/28 13:15:08 phintuka Exp $
  *
  */
 
+//#define LOG_UDP_RATE
+//#define LOG_RESEND
+//#define LOG_SCR
+
 #define __STDC_FORMAT_MACROS
 #define __STDC_CONSTANT_MACROS 
 #include <inttypes.h>
@@ -27,14 +31,17 @@
 #include "../config.h"             // configuration data
 #include "../xine_input_vdr_net.h" // frame headers and constants
 
+#include "ts.h"
 #include "pes.h"
 #include "udp_buffer.h"
-#include "udp_pes_scheduler.h"
 #include "time_pts.h"
 #include "cxsocket.h"
 #include "sap.h"  // SAP  - Session Announcement Protocol
 #include "sdp.h"  // SDP  - Session Description Protocol
 #include "rtcp.h" // RTCP
+#include "sys_cap.h"
+
+#include "udp_pes_scheduler.h"
 
 
 #ifdef LOG_RESEND
@@ -49,36 +56,44 @@
 #  define LOGSCR(x...)
 #endif
 
+/*
+ * constants
+ */
 
-const int MAX_QUEUE_SIZE      = 64;       // ~ 65 ms with typical DVB stream
-const int MAX_LIVE_QUEUE_SIZE = (64+60);  // ~ 100 ms with typical DVB stream
-const int HARD_LIMIT          = (4*1024); // ~ 40 Mbit/s === 4 Mb/s
+const uint    MAX_QUEUE_SIZE      = 64;       // ~ 65 ms with typical DVB stream
+const uint    MAX_LIVE_QUEUE_SIZE = (64+60);  // ~ 100 ms with typical DVB stream
+const uint    HARD_LIMIT          = (4*1024); // ~ 40 Mbit/s === 4 Mb/s
+
+const uint    MAX_BURST_BYTES     = 32768;    // 32 kb
+const uint    MAX_BURST_FRAMES    = 15;       // 15 UDP packets
+
+const uint    RTCP_MIN_INTERVAL   = 45000;    // 500 ms (pts units)
 
 // initial burst length after seek (500ms = ~13 video frames)
-const int64_t INITIAL_BURST_TIME  = (int64_t)(45000); // pts units (90kHz)
+const int64_t INITIAL_BURST_TIME  = INT64_C(45000);  // pts units (90kHz)
 
 // assume seek when when pts difference between two frames exceeds this (2,5 seconds)
-const int64_t JUMP_LIMIT_TIME = (int64_t)(5*90000/2);   // pts units (90kHz)
+const int64_t JUMP_LIMIT_TIME     = INT64_C(225000); // pts units (90kHz)
 
-const int RTCP_MIN_INTERVAL = 45000; // max. twice in second
-
-
-typedef enum {
-  eScrDetect,
-  eScrFromAudio,
-  eScrFromPS1,
-  eScrFromVideo
-} ScrSource_t;
+const uint    SCHEDULER_MIN_DELAY_MS = 3;
+const uint    SCHEDULER_MAX_DELAY_MS = 20;
 
+static inline int64_t abs64(int64_t val) { return val<0 ? -val : val; }
 
 cUdpScheduler::cUdpScheduler()
 {
 
   // Scheduler data
 
-  current_audio_vtime = 0;
-  current_video_vtime = 0;
-  MasterClock.Set(INT64_C(0));
+  m_ScrSource         = eScrDetect;
+  m_CurrentAudioVtime = 0;
+  m_CurrentVideoVtime = 0;
+  m_CurrentPcr        = 0;
+
+  m_MasterClock.Set(INT64_C(0));
+
+  m_BurstBytes  = 0;
+  m_BurstFrames = 0;
 
   m_Master = false;
   m_TrickSpeed = false;
@@ -92,7 +107,7 @@ cUdpScheduler::cUdpScheduler()
   m_LastRtcpTime = 0;
   m_Frames = 0;
   m_Octets = 0;
-  RtpScr.Set((int64_t)random());
+  m_RtpScr.Set((int64_t)random());
 
   m_fd_sap = -1;
 
@@ -109,8 +124,6 @@ cUdpScheduler::cUdpScheduler()
 
   // Thread
 
-  m_Running = 1;
-
   Start();
 }
 
@@ -118,8 +131,9 @@ cUdpScheduler::~cUdpScheduler()
 {
   m_Lock.Lock();
 
-  m_Running = 0;
+  Cancel(-1);
   m_Cond.Broadcast();
+  m_CondWait.Signal();
   m_Lock.Unlock();
 
   Cancel(3);
@@ -132,6 +146,13 @@ cUdpScheduler::~cUdpScheduler()
   delete m_BackLog;
 }
 
+void cUdpScheduler::Scheduler_Sleep(int ms)
+{
+  m_CondWait.Wait(ms);
+  m_BurstBytes = 0;
+  m_BurstFrames = 0;
+}
+
 bool cUdpScheduler::AddRtp(void) 
 {
   cMutexLock ml(&m_Lock);
@@ -325,19 +346,19 @@ int cUdpScheduler::Poll(int TimeoutMs, bool Master)
     return DEFAULT_POLL_SIZE;
   }
   
-  int limit = m_Master ? MAX_QUEUE_SIZE : MAX_LIVE_QUEUE_SIZE;
+  uint limit = m_Master ? MAX_QUEUE_SIZE : MAX_LIVE_QUEUE_SIZE;
   if(m_QueuePending >= limit) {
     uint64_t WaitEnd = cTimeMs::Now();
     if(TimeoutMs >= 0)
       WaitEnd += (uint64_t)TimeoutMs;
 
-    while(cTimeMs::Now() < WaitEnd &&
-	  m_Running &&
-	  m_QueuePending >= limit)	
+    while (cTimeMs::Now() < WaitEnd &&
+           Running() &&
+           m_QueuePending >= limit)
       m_Cond.TimedWait(m_Lock, 5);
   }
 
-  return max(limit - m_QueuePending, 0);
+  return limit > m_QueuePending ? limit - m_QueuePending : 0;
 }
 
 bool cUdpScheduler::Flush(int TimeoutMs)
@@ -352,9 +373,9 @@ bool cUdpScheduler::Flush(int TimeoutMs)
     if(TimeoutMs >= 0)
       WaitEnd += (uint64_t)TimeoutMs;
 
-    while(cTimeMs::Now() < WaitEnd &&
-	  m_Running &&
-	  m_QueuePending > 0)
+    while (cTimeMs::Now() < WaitEnd &&
+           Running() &&
+           m_QueuePending > 0)
       m_Cond.TimedWait(m_Lock, 5);
   }
   return m_QueuePending == 0;
@@ -368,6 +389,8 @@ void cUdpScheduler::Clear(void)
 
   m_QueuePending = 0;
   m_Cond.Broadcast();
+
+  m_ScrSource = eScrDetect;
 }
 
 void cUdpScheduler::Pause(bool On)
@@ -375,9 +398,9 @@ void cUdpScheduler::Pause(bool On)
   cMutexLock ml(&m_Lock);
 
   if(On)
-    MasterClock.Pause();
+    m_MasterClock.Pause();
   else
-    MasterClock.Resume();
+    m_MasterClock.Resume();
 
   m_TrickSpeed = false;
 }
@@ -395,7 +418,7 @@ void cUdpScheduler::TrickSpeed(const int Multiplier)
     LOGMSG("UDP clock --> 1/%d", Multiplier);
 #endif
 
-  MasterClock.TrickSpeed(Multiplier);
+  m_MasterClock.TrickSpeed(Multiplier);
 
   m_TrickSpeed = (Multiplier==-1 || Multiplier==1) ? false : true;
 }
@@ -404,22 +427,22 @@ void cUdpScheduler::SetScrSpeed(const int Speed)
 {
   cMutexLock ml(&m_Lock);
 
-  MasterClock.SetScrSpeed(Speed);
-  RtpScr.SetScrSpeed(Speed);
+  m_MasterClock.SetScrSpeed(Speed);
+  m_RtpScr.SetScrSpeed(Speed);
 }
 
-bool cUdpScheduler::Queue(uint64_t StreamPos, const uchar *Data, int Length) 
+bool cUdpScheduler::Queue(eStreamId StreamId, uint64_t StreamPos, const uchar *Data, int Length)
 {
   cMutexLock ml(&m_Lock);
 
-  if(m_Handles[0] < 0) 
+  if(m_Handles[0] < 0)
     return true;
 
-  int limit = m_Master ? MAX_QUEUE_SIZE : MAX_LIVE_QUEUE_SIZE;
+  uint limit = m_Master ? MAX_QUEUE_SIZE : MAX_LIVE_QUEUE_SIZE;
   if(m_QueuePending >= limit)
     return false;
 
-  m_BackLog->MakeFrame(StreamPos, Data, Length); 
+  m_BackLog->MakeFrame(StreamId, StreamPos, Data, Length);
   m_QueuePending++;
 
   m_Cond.Broadcast();
@@ -427,51 +450,90 @@ bool cUdpScheduler::Queue(uint64_t StreamPos, const uchar *Data, int Length)
   return true;
 }
 
-int cUdpScheduler::calc_elapsed_vtime(int64_t pts, bool Audio) 
+void cUdpScheduler::QueuePadding(void)
+{
+  cMutexLock ml(&m_Lock);
+
+  if (m_Handles[0] < 0)
+    return;
+
+  if (m_QueuePending > 2)
+    return;
+
+  QueuePaddingInternal();
+
+  m_Cond.Broadcast();
+}
+
+void cUdpScheduler::QueuePaddingInternal(void)
+{
+  static unsigned const char Padding[] = {0x00,0x00,0x01,0xBE,0x00,0x02,0xff,0xff};
+
+  int PrevSeq = (m_QueueNextSeq + UDP_BUFFER_SIZE - 1) & UDP_BUFFER_MASK;
+  stream_rtp_header_impl_t *Frame = m_BackLog->Get(PrevSeq);
+  if (Frame) {
+    int      PrevLen = m_BackLog->PayloadSize(PrevSeq);
+    uint64_t Pos     = ntohll(Frame->hdr_ext.pos) + PrevLen - 8;
+    m_BackLog->MakeFrame(sidPadding, Pos, Padding, 8);
+  } else
+    m_BackLog->MakeFrame(sidPadding, 0, Padding, 8);
+
+  m_QueuePending++;
+}
+
+int cUdpScheduler::CalcElapsedVtime(int64_t pts, ScrSource_t ScrSource)
 {
   int64_t diff = 0;
 
-  if(!Audio) {
-    diff = pts - current_video_vtime;
-    if(diff > JUMP_LIMIT_TIME || (-diff) > JUMP_LIMIT_TIME) { // 1 s (must be > GOP)
+  switch (ScrSource) {
+
+  case eScrFromVideo:
+    diff = pts - m_CurrentVideoVtime;
+    if (diff > JUMP_LIMIT_TIME || (-diff) > JUMP_LIMIT_TIME) { // 1 s (must be > GOP)
       // RESET
-#ifdef LOG_SCR
-      LOGDBG("cUdpScheduler RESET (Video jump %lld->%lld)",
-	     current_video_vtime, pts);
-#endif
-      current_video_vtime = pts;
+      LOGSCR("cUdpScheduler SCR RESET (Video jump %lld->%lld)", m_CurrentVideoVtime, pts);
+      m_CurrentVideoVtime = pts;
 
       // Use video pts for sync only in audioless trickspeeds
       // (audio has smaller, constant and increasing intervals)
-      if(m_TrickSpeed)
-	MasterClock.Set(current_video_vtime + INITIAL_BURST_TIME);
+      if (m_TrickSpeed)
+	m_MasterClock.Set(m_CurrentVideoVtime + INITIAL_BURST_TIME);
 
       return -1;
     }
-    if(diff < 0)  /* ignore small negative differences (B/P frames are sent out-of-order) */
+    if (diff < 0)  /* ignore small negative differences (B/P frames are sent out-of-order) */
       diff = 0;
     else
-      current_video_vtime = pts;
-    
-  } else if(Audio) {
-    diff = pts - current_audio_vtime;
-    if(diff < 0) diff = -diff;
-    if(diff > JUMP_LIMIT_TIME) { // 1 sec
-      // RESET
-#ifdef LOG_SCR
-      LOGDBG("cUdpScheduler RESET (Audio jump %lld->%lld)",
-	     current_audio_vtime, pts);
-#endif
-      current_audio_vtime = pts;
+      m_CurrentVideoVtime = pts;
+    break;
 
-      // Use audio pts for sync (audio has constant and increasing intervals)
-      MasterClock.Set(current_audio_vtime + INITIAL_BURST_TIME);
-      
+  case eScrFromAudio:
+    diff = abs64(pts - m_CurrentAudioVtime);
+    if (diff > JUMP_LIMIT_TIME) { // 1 sec
+      // RESET
+      LOGSCR("cUdpScheduler SCR RESET (Audio jump %lld->%lld)", m_CurrentAudioVtime, pts);
+      m_CurrentAudioVtime = pts;
+      m_MasterClock.Set(m_CurrentAudioVtime + INITIAL_BURST_TIME);
       return -1;
     }
-    current_audio_vtime = pts;
+    m_CurrentAudioVtime = pts;
+    break;
+
+  case eScrFromPcr:
+    diff = pts - m_CurrentPcr;
+    if (diff > JUMP_LIMIT_TIME || diff < 0) { // 1 sec
+      // RESET
+      LOGSCR("cUdpScheduler RESET (PCR jump %lld->%lld)", m_CurrentPcr, pts);
+      m_CurrentPcr = pts;
+      m_MasterClock.Set(m_CurrentPcr + INITIAL_BURST_TIME);
+      return -1;
+    }
+    m_CurrentPcr = pts;
+    break;
+
+  default: break;
   }
- 
+
   return (int) diff;
 }
 
@@ -480,7 +542,7 @@ void cUdpScheduler::Send_RTCP(void)
   if(!m_fd_rtcp.open())
     return;
 
-  uint64_t scr = RtpScr.Now();
+  uint64_t scr = m_RtpScr.Now();
 
   if(scr > (m_LastRtcpTime + RTCP_MIN_INTERVAL)) {
     uint8_t frame[2048], *content = frame;
@@ -549,143 +611,143 @@ void cUdpScheduler::Send_RTCP(void)
 
 void cUdpScheduler::Send_SAP(bool Announce)
 {
-  if(xc.remote_rtp_sap && m_fd_rtp.open()) {
-    char ip[64] = "";
-    uint32_t local_addr = m_fd_rtp.get_local_address(ip);
-    if(local_addr) {
-      const char *sdp_descr = vdr_sdp_description(ip,
-						  2001,
-						  xc.listen_port,
-						  xc.remote_rtp_addr,
-						  m_ssrc,
-						  xc.remote_rtp_port,
-						  xc.remote_rtp_ttl);
-#if 1
-      /* store copy of SDP data */
-      if(m_fd_sap < 0) {
-	cString fname = AddDirectory(VideoDirectory,
-				     cString::sprintf("xineliboutput@%s.sdp",
-						      ip));
-	FILE *fp = fopen(fname, "w");
-	if(fp) {
-	  fprintf(fp, "%s", sdp_descr);
-	  fclose(fp);
-	}
-      }
-#endif
-      sap_pdu_t *pdu = sap_create_pdu(local_addr,
-				      Announce, 
-				      (m_ssrc >> 16 | m_ssrc) & 0xffff,
-				      "application/sdp",
-				      sdp_descr);
-      
-      if(!sap_send_pdu(&m_fd_sap, pdu, 0))
-	LOGERR("SAP/SDP announce failed");
-      free(pdu);
+  if(!xc.remote_rtp_sap || !m_fd_rtp.open())
+    return;
 
-      if(!Announce)
-	CLOSESOCKET(m_fd_sap);
+  char ip[64] = "";
+  uint32_t local_addr = m_fd_rtp.get_local_address(ip);
+
+  if(!local_addr)
+    return;
+
+  uint32_t payload_type = VDRVERSNUM > 10702 ? SDP_PAYLOAD_MPEG_TS : SDP_PAYLOAD_MPEG_PES;
+  const char *sdp_descr = vdr_sdp_description(ip,
+                                              2001,
+                                              xc.listen_port,
+                                              xc.remote_rtp_addr,
+                                              payload_type,
+                                              m_ssrc,
+                                              xc.remote_rtp_port,
+                                              xc.remote_rtp_ttl);
+  if(!sdp_descr)
+    return;
+
+#if 1
+  /* store copy of SDP data */
+  if(m_fd_sap < 0) {
+    cString fname = AddDirectory(VideoDirectory,
+				 cString::sprintf("xineliboutput@%s.sdp",
+						  ip));
+    FILE *fp = fopen(fname, "w");
+    if(fp) {
+      fprintf(fp, "%s", sdp_descr);
+      fclose(fp);
     }
   }
+#endif
+
+  sap_pdu_t *pdu = sap_create_pdu(local_addr,
+				  Announce, 
+				  (m_ssrc >> 16 | m_ssrc) & 0xffff,
+				  "application/sdp",
+				  sdp_descr);
+      
+  if(!sap_send_pdu(&m_fd_sap, pdu, 0))
+    LOGERR("SAP/SDP announce failed");
+  free(pdu);
+
+  if(!Announce)
+    CLOSESOCKET(m_fd_sap);
 }
-  
+
+#ifdef LOG_SCR
+static const char ScrSourceName[][6] = {"???", "VIDEO", "PS1", "AUDIO", "PCR"};
+#endif
+
 void cUdpScheduler::Schedule(const uchar *Data, int Length)
 {
-  bool Audio = IS_AUDIO_PACKET(Data), Video = IS_VIDEO_PACKET(Data);
-  int64_t pts = PES_HAS_PTS(Data) ? pes_get_pts(Data, Length) : INT64_C(-1);
-  int elapsed = pts>0 ? calc_elapsed_vtime(pts, Audio) : 0;
+  int64_t     pts       = NO_PTS;
+  int         elapsed   = 0;
+  ScrSource_t ScrSource = eScrDetect;
+
+  // Get timestamp from data
+
+  if (DATA_IS_TS(Data)) {
+    if (ts_get_pcr_n(Data, Length/TS_SIZE, &pts)) {
+      LOGSCR("UDP PCR: %"PRId64, pts);
+      ScrSource = eScrFromPcr;
+      elapsed   = CalcElapsedVtime(pts, eScrFromPcr);
+    }
+  } else /* if (DATA_IS_PES(Data)) */ {
+    if (PES_HAS_PTS(Data)) {
+      ScrSource = IS_VIDEO_PACKET(Data) ? eScrFromVideo : IS_AUDIO_PACKET(Data) ? eScrFromAudio : eScrDetect;
+      pts       = pes_get_pts(Data, Length);
+      elapsed   = CalcElapsedVtime(pts, ScrSource);
+    }
+  }
+
+  if (m_ScrSource < ScrSource)
+    m_ScrSource = ScrSource;
 
   if(elapsed > 0) {
-    int64_t now = MasterClock.Now();
-    LOGSCR("PTS: %lld  (%s) elapsed %d ms (PID %02x)", 
-	   pts, Video?"Video":Audio?"Audio":"?", elapsed/90, Data[3]);
+
+    int64_t now      = m_MasterClock.Now();
+    int64_t SendTime = ScrSource==eScrFromVideo ? m_CurrentVideoVtime :
+                       ScrSource==eScrFromAudio ? m_CurrentAudioVtime :
+                       ScrSource==eScrFromPcr   ? m_CurrentPcr        : now;
+
+    LOGSCR("PTS: %lld  (%s) elapsed %d ms (PID %02x)", pts, ScrSourceStr[ScrSource],
+           pts_to_ms(elapsed), DATA_IS_TS(Data) ? TS_PID(Data) : Data[3]);
 
     //
-    // Detect discontinuity
+    // Detect discontinuity (against SCR)
     //
-    if(Audio) {
-      if(now > current_audio_vtime && (now - current_audio_vtime)>JUMP_LIMIT_TIME) {
-	LOGSCR("cUdpScheduler MasterClock init (was in past)");
-	MasterClock.Set(current_audio_vtime + INITIAL_BURST_TIME);
-      } else if(now < current_audio_vtime && (current_audio_vtime-now)>JUMP_LIMIT_TIME) {
-	LOGSCR("cUdpScheduler MasterClock init (was in future)");
-	MasterClock.Set(current_audio_vtime + INITIAL_BURST_TIME);
-      }
-    }
-
-    else if(Video && m_TrickSpeed) {
-      if(now > current_video_vtime && (now - current_video_vtime)>JUMP_LIMIT_TIME) {
-	LOGSCR("cUdpScheduler MasterClock init (was in past) - VIDEO");
-	MasterClock.Set(current_video_vtime + INITIAL_BURST_TIME);
-      } else if(now < current_video_vtime && (current_video_vtime-now)>JUMP_LIMIT_TIME) {
-	LOGSCR("cUdpScheduler MasterClock init (was in future) - VIDEO");
-	MasterClock.Set(current_video_vtime + INITIAL_BURST_TIME);
-      }
+    if (now > SendTime && (now - SendTime) > JUMP_LIMIT_TIME) {
+      LOGSCR("cUdpScheduler MasterClock init (was in past) %s", ScrSourceStr[ScrSource]);
+      now = SendTime + INITIAL_BURST_TIME;
+      m_MasterClock.Set(now);
+    } else if (now < SendTime && (SendTime - now) > JUMP_LIMIT_TIME) {
+      LOGSCR("cUdpScheduler MasterClock init (was in future) %s", ScrSourceStr[ScrSource]);
+      now = SendTime + INITIAL_BURST_TIME;
+      m_MasterClock.Set(now);
     }
 
     //
     // Delay
     //
-    int delay_ms = 0;
-    if(m_TrickSpeed ) {
-      if(current_video_vtime > now) {
-	delay_ms = (int)(current_video_vtime - now)/90;
-	LOGSCR("cUdpScheduler sleeping %d ms "
-	       "(time reference: %s, beat interval %d ms)",
-	       delay_ms, (Audio?"Audio PTS":"Video PTS"), elapsed/90);
-      }
-    } else {
-      if(current_audio_vtime > now) {
-	delay_ms = (int)(current_audio_vtime - now)/90;
-	LOGSCR("cUdpScheduler sleeping %d ms "
-	       "(time reference: %s, beat interval %d ms)",
-	       delay_ms, (Audio?"Audio PTS":"Video PTS"), elapsed/90);
-      }
-    }
-    while(delay_ms > 3) {
-      if(delay_ms > 20)
-	delay_ms = 20;
-      LOGSCR("  -> cUdpScheduler sleeping %d ms ", delay_ms);
-      CondWait.Wait(delay_ms);
-      now = MasterClock.Now();
-      delay_ms = (int)(current_video_vtime - now)/90;
+    while (SendTime > now) {
+
+      uint delay_ms = pts_to_ms(SendTime - now);
+      if (delay_ms < SCHEDULER_MIN_DELAY_MS)
+        break;
+
+      LOGSCR("cUdpScheduler sleeping %d ms (time ref: %s, beat interval %d ms)",
+             delay_ms, ScrSourceStr[ScrSource], pts_to_ms(elapsed));
+
+      if (delay_ms > SCHEDULER_MAX_DELAY_MS)
+        delay_ms = SCHEDULER_MAX_DELAY_MS;
+
+      Scheduler_Sleep(delay_ms);
+
+      now = m_MasterClock.Now();
     }
   }
 }
 
 void cUdpScheduler::Action(void)
 {
-#if 0
-  {
-    // Request real-time scheduling
-    sched_param temp;
-    temp.sched_priority = 2;
-
-    if (!pthread_setschedparam(pthread_self(), SCHED_RR, &temp)) {
-      LOGMSG("cUdpScheduler priority set successful SCHED_RR %d [%d,%d]",
-	     temp.sched_priority,
-	     sched_get_priority_min(SCHED_RR),
-	     sched_get_priority_max(SCHED_RR));
-    } else {
-      LOGMSG("cUdpScheduer: Can't set priority to SCHED_RR %d [%d,%d]",
-	     temp.sched_priority,
-	     sched_get_priority_min(SCHED_RR),
-	     sched_get_priority_max(SCHED_RR));
-    }
-  }
-#endif
-
   /* UDP Scheduler needs high priority */
   const int priority = -5;
-  SetPriority(priority);
+  if (have_cap_sys_nice())
+    SetPriority(priority);
   errno = 0;
   if ((nice(priority) == -1) && errno)
     LOGDBG("cUdpScheduler: Can't nice to value: %d", priority);
 
   m_Lock.Lock();
 
-  while(m_Running) {
-	
+  while (Running()) {
+
     if(m_Handles[0] < 0) {
       m_Cond.TimedWait(m_Lock, 5000); 
       continue;
@@ -693,23 +755,15 @@ void cUdpScheduler::Action(void)
 
     // Wait until we have outgoing data in queue
     if(m_QueuePending <= 0) {
+      m_BurstFrames = m_BurstBytes = 0;
       m_Cond.TimedWait(m_Lock, 100); 
       if(m_QueuePending <= 0) {
 	// Still nothing...
 	// Send padding frame once in 100ms so clients can detect 
 	// possible missing frames and server shutdown
-	static unsigned char padding[] = {0x00,0x00,0x01,0xBE,0x00,0x02,0xff,0xff};
-	int prevseq = (m_QueueNextSeq + UDP_BUFFER_SIZE - 1) & UDP_BUFFER_MASK;
-	stream_rtp_header_impl_t *frame = m_BackLog->Get(prevseq);
-	if(frame) {
-	  int prevlen = m_BackLog->PayloadSize(prevseq);
-	  uint64_t pos = ntohll(frame->hdr_ext.pos) + prevlen - 8;
-	  m_BackLog->MakeFrame(pos, padding, 8);
-	} else
-	  m_BackLog->MakeFrame(0, padding, 8);
-	m_QueuePending++;
+        QueuePaddingInternal();
       }
-      continue; // to check m_Running
+      continue; // to check Running()
     }
 
     // Take next frame from queue
@@ -726,38 +780,32 @@ void cUdpScheduler::Action(void)
     m_Lock.Unlock();
 
     // Schedule frame
-    if(m_Master)
+    if(m_Master && eStreamId(frame->hdr_ext.stream) == sidVdr)
       Schedule(frame->payload, PayloadSize);
 
-    // Need some bandwidth limit for ex. sequence of still frames when 
-    // moving cutting marks very fast (no audio or PTS available) 
-#if 1
-    // hard limit for used bandwidth:
-    // - ~1 frames/ms & 8kb/ms -> 8mb/s -> ~ 80 Mbit/s ( / client)
-    // - max burst 15 frames or 30kb
-    static int cnt = 0, bytes = 0;
-    static uint64_t dbg_timer = cTimeMs::Now();
-    static int dbg_bytes = 0;
-    cnt++; 
-    bytes += PayloadSize;
-    if(cnt>=15 && bytes >= 30000) {
-      CondWait.Wait(4);
+    // Need some bandwidth limit for ex. sequence of still frames when
+    // moving cutting marks very fast (no audio or PTS available).
+    // Hard limit for used bandwidth:
+    //   - ~1 frames/ms & 8kb/ms -> 8mb/s -> ~ 80 Mbit/s ( / client)
+    //   - max burst 15 frames or 32 kb
+    m_BurstFrames ++;
+    m_BurstBytes += PayloadSize;
+    if (m_BurstFrames >= MAX_BURST_FRAMES && m_BurstBytes >= MAX_BURST_BYTES) {
+      Scheduler_Sleep(4);
+#ifdef LOG_UDP_RATE
+      static uint64_t dbg_timer = cTimeMs::Now();
+      static int      dbg_bytes = 0;
       dbg_bytes += bytes;
-      cnt = 0; 
-      bytes = 0;
-      if(dbg_timer+60000 <= cTimeMs::Now()) {
-# if 0
-	LOGDBG("UDP rate: %4d Kbps (queue %d)", dbg_bytes/(60*1024/8),
-	       m_QueuePending);
-# endif
+      if (dbg_timer + 60000 <= cTimeMs::Now()) {
+	LOGDBG("UDP rate: %4d Kbps (queue %d)", dbg_bytes/(60*1024/8), m_QueuePending);
 	dbg_bytes = 0;
 	dbg_timer = cTimeMs::Now();
       }
-    }
 #endif
+    }
 
     /* tag frame with ssrc and timestamp */
-    frame->rtp_hdr.ts   = htonl((uint32_t)(RtpScr.Now() & 0xffffffff));
+    frame->rtp_hdr.ts   = htonl((uint32_t)(m_RtpScr.Now() & 0xffffffff));
     frame->rtp_hdr.ssrc = htonl(m_ssrc);
 
     /* deliver to all active sockets */
@@ -770,18 +818,18 @@ void cUdpScheduler::Action(void)
       // - kernel silently drops frames it cant send
       // -> poll() + send() just causes frames to be dropped
       //
-      int size = 0;
+      uint size = 0;
       if(!ioctl(m_Handles[i], TIOCOUTQ, &size)) {
 	if(size >= (m_wmem[i] - 2*RtpPacketLen)) {
 	  LOGMSG("cUdpScheduler: kernel transmit queue > ~%dkb (max %dkb) ! (master=%d)", 
 		 (m_wmem[i] - 2*RtpPacketLen)/1024, m_wmem[i]/1024, m_Master);
-	  CondWait.Wait(2);
+	  Scheduler_Sleep(2);
 	}
       }	else {
 	if(m_QueuePending > (MAX_QUEUE_SIZE-5))
 	  LOGDBG("cUdpScheduler: kernel transmit queue > ~30kb ! (master=%d ; Queue=%d)", 
 		 m_Master, m_QueuePending);
-	CondWait.Wait(2);
+	Scheduler_Sleep(2);
       }
 
       if(m_Handles[i] == m_fd_rtp.handle()) {
@@ -789,9 +837,7 @@ void cUdpScheduler::Action(void)
 	  LOGERR("cUdpScheduler: UDP/RTP send() failed !");
       } else {
 	/* UDP: send without rtp header */
-	if(send(m_Handles[i], 
-		((uint8_t*)frame) + sizeof(stream_rtp_header_impl_t) - sizeof(stream_udp_header_t), 
-		UdpPacketLen, 0) <= 0)
+	if (send(m_Handles[i], RTP_UDP_PAYLOAD(frame), UdpPacketLen, 0) <= 0)
 	  LOGERR("cUdpScheduler: UDP send() failed !");
       }
     }
@@ -821,9 +867,10 @@ void cUdpScheduler::ReSend(int fd, uint64_t Pos, int Seq1, int Seq2)
   if(fd < 0) /* no re-send for RTP */
     return;
 
-  char udp_ctrl[64] = {0};
-  ((stream_udp_header_t *)udp_ctrl)->seq = (uint16_t)(-1);
-  ((stream_udp_header_t *)udp_ctrl)->pos = (uint64_t)(-1);
+  struct {
+    stream_udp_header_t hdr;
+    char                payload[64-sizeof(stream_udp_header_t)];
+  } udp_ctrl = {{(uint64_t)INT64_C(-1), (uint16_t)-1, 0, {}}, {0}};
 
   // Handle buffer wrap
   if(Seq1 > Seq2)
@@ -832,36 +879,34 @@ void cUdpScheduler::ReSend(int fd, uint64_t Pos, int Seq1, int Seq2)
   cMutexLock ml(&m_Lock); // keeps also scheduler thread suspended ...
 
   if(Seq2-Seq1 > 64) {
-    LOGDBG("cUdpScheduler::ReSend: requested range too large (%d-%d)",
-	   Seq1, Seq2);
-
-    sprintf((udp_ctrl+sizeof(stream_udp_header_t)),
-	    "UDP MISSING %d-%d %" PRIu64,
-	    Seq1, (Seq2 & UDP_BUFFER_MASK), Pos);
-    send(fd, udp_ctrl, sizeof(udp_ctrl), 0);
+    LOGDBG("cUdpScheduler::ReSend: requested range too large (%d-%d)", Seq1, Seq2);
+    snprintf(udp_ctrl.payload, sizeof(udp_ctrl.payload),
+             "UDP MISSING %d-%d %" PRIu64,
+             Seq1, (Seq2 & UDP_BUFFER_MASK), Pos);
+    send(fd, &udp_ctrl, sizeof(udp_ctrl), 0);
     return;
   }
 
   // re-send whole range
   for(; Seq1 <= Seq2; Seq1++) {
-      
+
     // Wait if kernel queue is full
     int size = 0;
     if(!ioctl(fd, TIOCOUTQ, &size))
       if(size > ((0x10000)/2 - 2048)) { // assume 64k kernel buffer
 	LOGDBG("cUdpScheduler::ReSend: kernel transmit queue > ~30kb !");
-	cCondWait::SleepMs(2);
+	Scheduler_Sleep(2);
       }
-    
+
     stream_rtp_header_impl_t *frame = m_BackLog->Get(Seq1);
-      
+
     if(frame) {
       if(ntohull(frame->hdr_ext.pos) - Pos < 100000) {
-	send(fd, 
-	     ((uint8_t*)frame) + sizeof(stream_rtp_header_impl_t) - sizeof(stream_udp_header_t), 
-	     m_BackLog->PayloadSize(Seq1) + sizeof(stream_udp_header_t), 
+	send(fd,
+	     RTP_UDP_PAYLOAD(frame),
+	     m_BackLog->PayloadSize(Seq1) + sizeof(stream_udp_header_t),
 	     0);
-	LOGRESEND("cUdpScheduler::ReSend: %d (%d bytes) @%lld sent", 
+	LOGRESEND("cUdpScheduler::ReSend: %d (%d bytes) @%lld sent",
 		  Seq1, m_BackLog->PayloadSize(Seq1), Pos);
 	Pos = ntohull(frame->hdr_ext.pos) + m_BackLog->PayloadSize(Seq1);
 	continue;
@@ -877,21 +922,19 @@ void cUdpScheduler::ReSend(int fd, uint64_t Pos, int Seq1, int Seq2)
     // buffer has been lost - send packet missing info
 
     LOGRESEND("cUdpScheduler::ReSend: missing %d-%d @%d (hdr 0x%llx 0x%x)",
-	   Seq1, Seq1, Pos,
-	   ((stream_udp_header_t *)udp_ctrl)->pos,
-	   ((stream_udp_header_t *)udp_ctrl)->seq);
+              Seq1, Seq1, Pos, udp_ctrl.hdr.pos, udp_ctrl.hdr.seq);
 
     int Seq0 = Seq1;
-    for(; Seq1 <= Seq2; Seq1++) {
+    for(; Seq1 < Seq2; Seq1++) {
       stream_rtp_header_impl_t *frame = m_BackLog->Get(Seq1+1);
       if(frame && (ntohull(frame->hdr_ext.pos) - Pos < 100000))
 	break;
     }
 
-    sprintf((udp_ctrl+sizeof(stream_udp_header_t)),
-	    "UDP MISSING %d-%d %" PRIu64,
-	    Seq0, (Seq1 & UDP_BUFFER_MASK), Pos);
+    snprintf(udp_ctrl.payload, sizeof(udp_ctrl.payload),
+             "UDP MISSING %d-%d %" PRIu64,
+             Seq0, (Seq1 & UDP_BUFFER_MASK), Pos);
 
-    send(fd, udp_ctrl, sizeof(udp_ctrl), 0);
+    send(fd, &udp_ctrl, sizeof(udp_ctrl), 0);
   }
 }
